#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDALApplication.py 706 2009-05-13 10:35:52Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from ALDefinition               import *
from ALApplication              import ALApplication
from ALAppDefinitionInit        import ALAppDefinitionInit
from ALAppDefinitionEnd         import ALAppDefinitionEnd
from ALApplicationActionControl import ALApplicationActionControl
from ALDebug                    import ALDebug
from ALSystem                   import ALSystem

# Create ImportAALib To avoid circular imports
from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDTime

# For Bug#10 workaround
from EDImportLib             import EDCommandLine
from EDImportLib             import EDCommandLineArgument
from EDImportLib             import EDAppDefinition
from EDImportLib             import EDResource
from EDImportLib             import EDTime
from EDImportLib             import EDLogFile
from EDImportLib             import EDSlot
from EDImportLib             import EDSignal
from EDImportLib             import EDDebug
from EDImportLib             import EDDiskExplorer
# End Bug#10 workaround

from EDConfiguration import EDConfiguration


class EDALApplication( ALApplication ):

    """
    This class is the ALApplication with specific needs for EDNA
    """
    def __init__( self, _edStringName, _edStringVersion, _edLogFile = None ):
        """
        """
                  
        self.m_edLogFile = _edLogFile
        
        ALApplication.__init__( self, _edStringName, _edStringVersion )

            
    def init( self, _bInit = True ):
        """
        This is the Init method in the ALAppDefinitionInit.
        """
        EDVerbose.DEBUG( "Execution: ALAppDefinitionInit.init()" )
        oBoolStatus = True
        edString = EDString( ALAppDefinitionInit.m_oalCommandLine.getCommandLine() )
        EDVerbose.log( edString )
        EDVerbose.setVerboseOff()

        if (ALDebug.isDebugModeVerboseOn()==True):
            EDVerbose.setVerboseOn()

        if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--quiet" )):
            EDVerbose.setVerboseOff()

        if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--profile" )):
            ALAppDefinitionInit.setDebugProfileOn()

        if (ALDebug.isDebugModeOn()==True or ALDebug.isDebugModeForceOn()==True):
            ALAppDefinitionInit.setDebugMode()
            ALAppDefinitionInit.setDebugProfileOn()
            EDVerbose.DEBUG( "Debug Mode [ON]" )

        if (ALDebug.isDebugModeAssertOn()==True ):
            ALAssert.setAssertionModeOn()

        if (_bInit==True):
            EDVerbose.screen( self.m_oalApplicationName + "::Starting" )
            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--help" )
                or ALAppDefinitionInit.m_oalCommandLine.existCommand( "-h" )):
                EDVerbose.setVerboseOn()
                #ALAppDefinitionInit.outputHelp()
                self.usage()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--version" ) or
                ALAppDefinitionInit.m_oalCommandLine.existCommand( "-v" )):
                EDVerbose.setVerboseOn()
                ALAppDefinitionInit.outputVersion()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--man" ) or
                ALAppDefinitionInit.m_oalCommandLine.existCommand( "-m" )):
                EDVerbose.setVerboseOn()
                ALAppDefinitionInit.outputMan()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--no-log" )):
                EDVerbose.setLogFileOff()

            # workaround to avoid problem of the ALString.toXml cleaning function of jython
            if (ALSystem.getOSName()=="Linux" and ALSystem.getMachineVirtuel()=="Java"):
                pass
            else:
                oalObject = ALAppDefinitionInit.getResource( AL_FRAMEWORK_LAST_RUN )
                if (oalObject != None):
                    edStringLastRun = EDString( oalObject );
                    EDVerbose.log( "Last Run: " + edStringLastRun )

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--import-resource" )):
                oalStrResourceFileName = ALAppDefinitionInit.m_oalCommandLine.getCommandArgument( "--import-resource" )
                ALAppDefinitionInit.importResource( oalStrResourceFileName )

        # The OK to continue the application
        return oBoolStatus
    

    def end( self ):
        """
        End The application framework...closing and registering the resources
        """
        self.__forceCloseActions()
        self.__forceCloseTimerActions()
        ALAppDefinitionEnd.setResource( AL_FRAMEWORK_LAST_RUN, EDTime.timeString() )
        ALAppDefinitionEnd.outputResource()
        self.__getActionLog()


        ofAppRunTime     = self.getRunTime()
        oalStrAppName    = ALAppDefinitionEnd.getApplicationName()
        edStrAppRunTime = EDString( ofAppRunTime )
        EDVerbose.outputActionLog()
        EDVerbose.log( oalStrAppName + "::Closing" )
        EDVerbose.screen( oalStrAppName + "::Runtime: " + edStrAppRunTime + " [s]" )
        EDVerbose.log( oalStrAppName + "::Runtime: " + EDTime.getTimeStringPSP( ofAppRunTime ) )


        # Used to generate a XML profile log file to analise the performance
        ALAppDefinitionEnd.outputProfile()

        # To be safety and close external pipes on remote control
        #ALSystem.exit( 0 ) #Bug in Python 2.4 - it raises an exception !!!!!!!!!!


    def __forceCloseTimerActions( self ):
        """
        This ALAppDefinitionEnd force Close Timer Actions.
        """
        EDVerbose.DEBUG( "Execution: ALAppDefinitionEnd.__forceCloseTimerActions()" )
        ALAppDefinitionEnd.synchronizeOn()
        for iAction in ALAppDefinitionEnd.m_oalListTimerActions:
            iAction.close()
        ALAppDefinitionEnd.synchronizeOff()


    def __forceCloseActions( self ):
        """
        This ALAppDefinitionEnd force Close All Actions.
        """
        EDVerbose.DEBUG( "Execution: ALAppDefinitionEnd.__forceCloseActions()" )
        ALAppDefinitionEnd.synchronizeOn()
        ALApplicationActionControl.forceClose()
        ALAppDefinitionEnd.synchronizeOff()


    def __getActionLog( self ):
        ALApplicationActionControl.getActionLogInformation()
        
        
    def create( self, _obPlugin = True ):
        """
        The following code is related to bug#10 workaround
        """
        
        EDAppDefinition.m_oalCommandLine          = EDCommandLine( EDCommandLineArgument() )
        EDAppDefinition.m_oalApplicationFolder    = EDAppDefinition.m_oalCommandLine.getApplicationFolder()
        
        if( self.m_edLogFile == None ):
            EDVerbose.m_oalLogFile                    = EDLogFile( EDAppDefinition.m_oalApplicationName + ".log" )
        else:
            EDVerbose.m_oalLogFile                    = self.m_edLogFile
            
        EDAppDefinition.m_oalResource             = EDResource( EDAppDefinition.m_oalApplicationName + ".res" )
        self.m_oalTime = EDTime()
        self.m_ofTimeOut = AL_TIME_INFINITE # It is: 11d:13h:46m:39s:000ms

        # Only for static methods
        EDTime.init()

        self.m_oalSlotInit          = EDSlot()
        self.m_oalSlotPreProcess    = EDSlot()
        self.m_oalSlotProcess       = EDSlot()
        self.m_oalSlotPostProcess   = EDSlot()
        self.m_oalSlotEnd           = EDSlot()

        EDAppDefinition.m_oalSlotSignal.connect( self.exit, EDSignal.error() )
        EDAppDefinition.m_oalSlotSignal.connect( self.exit, EDSignal.exception() )
        EDAppDefinition.m_oalSlotSignal.connect( self.exit, EDSignal.exit() )
        EDAppDefinition.m_oalCommandLine.m_oalSlotException.connect( self.exit, EDSignal.exception() ) # not used??

