#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDFactoryPlugin.py 744 2009-05-29 08:53:14Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal author:    Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author: Marie-Francoise Incardona (incardon@esrf.fr)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDObject
from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList
from EDImportLib import EDDictionary

from EDImportSystem import PyOs
from EDImportSystem import PySys

from EDUtilsPath import EDUtilsPath


class EDFactoryPlugin( EDObject ):
    """
    This class provides a factory for loading plugins and/or modules. By default all plugins/modules located
    in $EDNA_HOME can be loaded with this class. A plugin or module located elsewhere can be loaded
    provided one of its parent directories is added with the method "addPluginRootDirectory".
    
    The plugin or module must have file name which starts with "EDPlugin" or "XSData", and the suffix ".py".
    The following two directories are not searched for plugins in order to improve performance:
    $EDNA_HOME/libraries and $EDNA_HOME/prototype.
    
    Both the loadPlugin and loadModule methods will automatically add the module location to the
    application python path, if it not already exists on the path. 

    The EDFactoryPlugin also adds the path to a "src" directory if the following scheme for
    locating the plugins is followed:
    
    EDNA application, e.g. mxv1, or plugin collection e.g. mxEcecPlugins
     |
     |-datamodel
     |-src
     |-tests
     |   |-data
     |   |-testsuite
     |
     |-plugins
     |   |-EDPlugin[name of plugin]-v1.0
     |   |   |-plugins
     |   |   |   |-EDPlugin[name of plugin]v10.py
     |   |   |   |-EDPlugin[another name of plugin]v10.py
     
     The "src" directory above is used for storing the data bindings for a common data model,
     e.g. XSDataMXv1.py, or in general for code used by all the plugins. The path to the "src"
     directory is added using a relative path from the plugin location: ../../../src
     The path to the src directory is only added if the src directory exists.
    """
    
    def __init__( self ):
        self.m_edListPluginRootDirectory  = None
        self.m_edDictionaryModuleLocation = None
        # The following two paths are excluded from the search of plugins and modules
        # in order to gain speed:
        self.m_edStringPathToPrototype = EDUtilsPath.mergePath( EDUtilsPath.getEdnaHome(), "prototype" )
        self.m_edStringPathToLibraries = EDUtilsPath.mergePath( EDUtilsPath.getEdnaHome(), "libraries" )
        # The handling of the paths to exclude should be improved, see bug #306


    def getModuleLocation( self, _edStringPluginName ):
        """
        This method returns the location of a module, e.g. XSDataCommon.
        """
        edStringModuleLocation = None
        if ( self.m_edDictionaryModuleLocation is None ):
            self.addPluginRootDirectory( EDUtilsPath.getEdnaHome() )
        if ( _edStringPluginName in self.m_edDictionaryModuleLocation.keys() ):
            edStringModuleLocation = self.m_edDictionaryModuleLocation[ _edStringPluginName ]
        return edStringModuleLocation
    
        
    def isPlugin( self, _edStringFileName ):
        """
        This method returns a True if the file name provided is considered to
        be an EDNA plugin or module, i.e. it starts with either "EDPlugin" or "XSData" 
        and it ends with ".py", otherwise the method returns False.
        """
        bValue = False
        if ( ( _edStringFileName.startswith( "EDPlugin" ) or _edStringFileName.startswith( "XSData" ) ) and \
             _edStringFileName.endswith( ".py" ) ):
            bValue = True
        return bValue
        
        
    def __addPluginLocation( self, _edStringPluginRootDirectory, _pyStrDirectoryVisit, _pyListDirectory  ):
        """
        This method is called by the python walk command in the addPluginRootDirectory method.
        
        If the _pyStrDirectoryVisit path starts with the path to the prototype or the libraries
        the method returns without any further actions.
        
        It checks all the file names in the _pyListDirectory list if they corresponds to plugins or modules
        using the method "isPlugin". If the file name corresponds to a plugin or module, the location of the
        plugin or module is added to the self.m_edDictionaryModuleLocation.        
        """
        # Check if path is in prototype or in libraries, and if this is the case
        # exclude it in order to gain speed:
        if ( self.m_edStringPathToLibraries is not None ):
            if ( _pyStrDirectoryVisit.startswith( self.m_edStringPathToLibraries ) ):
                return
        if ( self.m_edStringPathToPrototype is not None ):
            if ( _pyStrDirectoryVisit.startswith( self.m_edStringPathToPrototype ) ):
                return
        if ( self.m_edDictionaryModuleLocation is None ):
            self.m_edDictionaryModuleLocation = EDDictionary()
        for pyStrFileName in _pyListDirectory:
            edStringFileName = EDString( pyStrFileName )
            if ( self.isPlugin( edStringFileName ) ):
                # Strip off the ".py" exctension
                edStringPluginName = edStringFileName[:-3]
                if ( edStringPluginName in self.m_edDictionaryModuleLocation.keys() ):
                    EDVerbose.warning( " *** EDFactoryPlugin: Found multiple plugins/modules with the same name!" )
                    EDVerbose.warning( " *** EDFactoryPlugin: Plugin/module already loaded:" + edStringPluginName + ", location: " + self.m_edDictionaryModuleLocation[ edStringPluginName ] )
                    EDVerbose.warning( " *** EDFactoryPlugin: Duplicate plugin/module definition in :" + _pyStrDirectoryVisit )
                else: 
                    self.m_edDictionaryModuleLocation[ edStringPluginName ] = EDString( _pyStrDirectoryVisit )
            
    
    def addPluginRootDirectory( self, _edStringPluginRootDirectory ):
        """
        This method can be called by an application if plugins or modules are supposed to be loaded
        outside the EDNA_HOME directory.
        """
        if ( self.m_edListPluginRootDirectory is None ):
            self.m_edListPluginRootDirectory = EDList()
        self.m_edListPluginRootDirectory.add( _edStringPluginRootDirectory )
        # The following Python call will be deprecated in Python 3.0, see bug #262
        PyOs.path.walk( _edStringPluginRootDirectory, self.__addPluginLocation, _edStringPluginRootDirectory )
    
    
    def loadPlugin( self, _edStringPluginName ):
        """
        This method loads a plugin if it's present in the self.m_edDictionaryModuleLocation.
        """
        edPlugin = None
        if ( self.m_edListPluginRootDirectory is None ):
            self.addPluginRootDirectory( EDUtilsPath.getEdnaHome() )
        if ( _edStringPluginName in self.m_edDictionaryModuleLocation.keys() ):
            edStringModuleLocation = self.m_edDictionaryModuleLocation[ _edStringPluginName ]
            self.appendPath( edStringModuleLocation )
            oModule = __import__( _edStringPluginName )
            edPlugin = oModule.__dict__[ _edStringPluginName ]()
        else:
            EDVerbose.warning( "Plugin not found: " + _edStringPluginName )
        return edPlugin
    
   
    def loadModule( self, _edStringModuleName ):
        """
        This method loads a module, i.e. it adds the module to the python path.
        """
        if ( self.m_edListPluginRootDirectory is None ):
            self.addPluginRootDirectory( EDUtilsPath.getEdnaHome() )
        if ( _edStringModuleName in self.m_edDictionaryModuleLocation.keys() ):
            edStringModuleLocation = self.m_edDictionaryModuleLocation[ _edStringModuleName ]
            self.appendPath( edStringModuleLocation )
        
    
    
    def appendPath( self, _edStringModuleLocation ):
        """
        This method appends the plugin "src" directory to the system path, if it's not already present.
        """
        if ( not _edStringModuleLocation in PySys.path ):
            PySys.path.append( _edStringModuleLocation )
            edStringSrcDirectory = EDUtilsPath.appendListOfPaths( _edStringModuleLocation, [ "..", "..", "..", "src" ] )
            if ( PyOs.path.exists( edStringSrcDirectory ) ):
                if ( not edStringSrcDirectory in PySys.path ):
                    PySys.path.append( edStringSrcDirectory )


