#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDPlugin.py 703 2009-05-13 09:46:41Z incardon $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#

from EDImportLib import EDInformation
from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDSlot
from EDImportLib import EDList
from EDImportLib import EDTime
from EDImportLib import EDRandom
from EDImportLib import EDDiskExplorer

from EDImportSystem import  PyOs

from EDApplication   import EDApplication
from EDConfiguration import EDConfiguration
from EDMessage       import EDMessage
from EDUtilsFile     import EDUtilsFile

from ALPlugin import ALPlugin

from XSDataCommon import XSPluginItem

# To be removed ASAP (see Bug#184)
#from EDApplicationDeprecated import EDApplicationDeprecated


class EDPlugin( ALPlugin ):
    """
    This is the EDNA plugin main class
    An EDNA plugin class:
    - is a configurable entity
    - has a base name (<date>-<random number>-<base name>)
    - handles input/output data (setter, getter, checker)
    - has warning and error messages
    - has a base and a working directory (both are configurable)
        The working directory is the folder from which the plugin is launched
        and should contain all associated files with the plugin execution (edna xml input/output, 3rd party output files)
        The base directory is the parent directory of the working directory. 
        Example: the working directory of a control plugin is the base directory of the plugins that it invokes,
        i.e. the plugins working directories has the control plugin working directory as parent.
    - defines the method that generates an executive summary (user-related output summary) that sub-classes should implement
    """
    
    def __init__ ( self ):
        """
        Initializes plugin related attributes described above
        """
        ALPlugin.__init__( self )
        edInformation = EDInformation( "EDNA Developers Team", 
                                        "0.1",
                                        "Developed with the EDNA Framework", 
                                        "http://www.edna-site.org",
                                        "Under GPL v3 License" )
        edInformation.setGroupName( self.getGroup() )
        edInformation.setFileName( self.getSource() )
        self.setInformation( edInformation )
        self.m_xsPluginItem = None
        self.m_xsDataInputClass = None
        self.m_xsDataInput = None
        self.m_xsDataStrXmlInput = None
        self.m_xsDataOutput = None
        self.m_edSlotExportDataOutput = EDSlot()
        self.m_edStringExecutionContext = "EDApplication"
        
        self.m_edListErrorMessages = EDList()
        self.m_edListWarningMessages = EDList()
        
        # The handling of the base directory should be moved to EDPluginExec
        self.m_edStrCONF_BASE_DIR_LABEL       = "baseDirectory"
        self.m_edStringBaseDirectory          = None

        self.m_edStringCONF_WORKING_DIR_LABEL    = "workingDirectory"
        self.m_edStringWorkingDirectory       = None

        self.m_edStringBaseName = None
        
        self.m_edListExecutiveSummaryLines = EDList()
        self.m_edStringExecutiveSummarySeparator = "-----------------------------------------------------------------------------------------------------------------"
        # The m_oedStringExecutiveSummarySeparator is deprecated, and should be removed once the EDPluginControlStartegyv01 has been updated.
        self.m_oedStringExecutiveSummarySeparator = self.m_edStringExecutiveSummarySeparator
        
        
    def preProcess( self, _edObject = None ):
        """
        Connect main methods responsible for:
            - writing xml data input/output in the working dir
            - generating the executive summary after the plugin execution
        Initialize the base directoy
        Configures the plugin
        Checks the input data
        """
        EDVerbose.DEBUG( "*** EDPlugin.preProcess")
        self.connectPostProcess( self.exportDataOutput )
        self.connectPreProcess( self.writeDataInput )
        self.connectPostProcess( self.writeDataOutput )
        self.connectPostProcess( self.generateExecutiveSummary )
        
        if( self.m_edStringExecutionContext == "EDApplication"):
            pluginConfiguration = None
            # To be removed ASAP (see Bug#184)
#            if( EDApplication.getApplicationName() == "EDApplicationPrototype-v1.0.0" ):
#                pluginConfiguration = EDApplicationDeprecated.getPluginConfiguration( self.getClassName() )
#            else:
            # end To be removed ASAP (see Bug#184)
            pluginConfiguration = EDApplication.getPluginConfiguration( self.getClassName() )
            if ( pluginConfiguration == None ):
                EDVerbose.DEBUG( "*** EDPlugin.preProcess: pluginConfiguration is None for "+ self.getClassName())            
            self.setConfiguration( pluginConfiguration )
            
        if ( self.m_edStringBaseName == None ):
            self.setBaseName( self.createBaseName() )
        self.configure()         
        self.checkParameters()
           
    
    def setConfiguration( self, _xsPluginItem ):
        """
        Receives an XSPluginItem (Plugin Configuration) from the application
        """
        EDVerbose.DEBUG( "*** EDPlugin.setConfiguration")
        self.m_xsPluginItem = _xsPluginItem
        
        
    def getConfiguration( self ):
        """
        Gets the Plugin Configuration as an XSPluginItem
        """
        EDVerbose.DEBUG( "*** EDPlugin.getConfiguration")
        return self.m_xsPluginItem
    
    
    def configure( self ):
        """
        Should be overridden by the Final Plugin If needed
        This method should set its proper members attributes from a Plugin configuration Object
        """
        EDVerbose.DEBUG( "*** EDPlugin.configure")
        xsPluginItem = self.getConfiguration()
        if ( xsPluginItem is None ):
            EDVerbose.DEBUG( "*** EDPlugin.configure: No plugin item defined." )
            xsPluginItem = XSPluginItem()
        # Base directory
        edStringBaseDirectory = self.getBaseDirectory()
        if ( edStringBaseDirectory is None):
            # Try to get base directory from plugin configuration
            edStringBaseDirectory = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStrCONF_BASE_DIR_LABEL )
            if( edStringBaseDirectory is None ):
                # Try to get working directory from environment variable
                edStringBaseDirectory = PyOs.environ.get("EDNA_BASE_DIRECTORY")
                if ( edStringBaseDirectory is None):
                    EDVerbose.DEBUG( "*** EDPlugin.configure: Using current base directory as working directory." )
                    edStringBaseDirectory = EDDiskExplorer.getCwd()
                else:
                    EDVerbose.DEBUG( "*** EDPlugin.configure: Setting base directory from $EDNA_WORKING_DIRECTORY." )
            else:
                if ( EDString( edStringBaseDirectory ).equals( "." ) ):
                    EDVerbose.DEBUG( "*** EDPlugin.configure: Using current base directory as working directory." )
                    edStringBaseDirectory = EDDiskExplorer.getCwd()
                else:
                    edStringBaseDirectory = EDDiskExplorer.getAbsolutePath( edStringBaseDirectory )
                    EDVerbose.DEBUG( "*** EDPlugin.configure: Setting base directory from plugin configuration." )
            self.setBaseDirectory( edStringBaseDirectory )
        else:
            EDVerbose.DEBUG( "*** EDPlugin.configure: Base directory already set before plugin is configured." )          
        # Working directory
        edStringWorkingDirectory = self.getWorkingDirectory()
        if ( edStringWorkingDirectory is None):
            # Try to get working directory from plugin configuration
            edStringWorkingDirectory = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStringCONF_WORKING_DIR_LABEL )
            if( edStringWorkingDirectory is not None ):
                EDVerbose.DEBUG( "*** EDPlugin.configure: Setting working directory from plugin configuration." )
            else:    
                EDVerbose.DEBUG( "*** EDPlugin.configure: Setting working directory as base directory + base name." )
                edStringWorkingDirectory = EDDiskExplorer.mergePath( self.getBaseDirectory(), self.getBaseName() )
            self.setWorkingDirectory( edStringWorkingDirectory )
        else:
            EDVerbose.DEBUG( "*** EDPlugin.configure: Working directory already set before plugin is configured." )          
        
        
    def checkParameters( self ):
        """
        Should be overridden by the Final Plugin If needed
        This method should check that the data input are consistent
        """
        EDVerbose.DEBUG( "*** EDPlugin.checkParameters")
        
        if( self.m_xsDataInputClass == None ):
            errorMessage = EDMessage.ERROR_MANDATORY_PARAM_MISSING_02 % ( self.getPluginName() + '.checkMandatoryParameters', "Data Input Class Not Defined !" )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )    
            raise RuntimeError, errorMessage 
    

    def setXSDataInputClass( self, _xsDataInputClass ):
        """
        This method should be called in the constructor of the derived plugins
        in order to set the XSData type of the input data, e.g. XSDataInputXXX
        """
        self.m_xsDataInputClass = _xsDataInputClass
            

    def getXSDataInputClass( self ):
        """
        Returns the XSData type of the input data.
        """
        return self.m_xsDataInputClass

    
    def setDataInput( self, _oDataInput ):
        """
        Sets the plugin input data.
        _oDataInput could be either an String XML or an XSData object.
        """
        EDVerbose.DEBUG( "*** EDPlugin.setDataInput" )
        # Allow for None input
        if ( _oDataInput is None ):
            EDVerbose.DEBUG( "*** EDPlugin.setDataInput: Data input is None" )
        elif ( self.getXSDataInputClass() is None ):
            edStringError = "No input XSData type defined for plugin %s" % self.getPluginName()
            errorMessage = EDMessage.ERROR_DATA_INPUT_CLASS_NOT_DEFINED_02 % ( self.getPluginName() + ".setDataInput", edStringError )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )   
            raise RuntimeError, errorMessage     
        else:
            # Check the type
            if( isinstance( _oDataInput, EDString ) ):
                EDVerbose.DEBUG( "*** EDPlugin.setDataInput: Input Data is of EDString " )
                self.m_xsDataInput = self.getXSDataInputClass().parseString( _oDataInput )
            elif ( isinstance( _oDataInput, self.getXSDataInputClass() ) ):
                self.m_xsDataInput = _oDataInput
            else:
                errorMessage = EDMessage.ERROR_WRONG_DATA_TYPE_02 % ( self.getPluginName(), 'setDataInput' )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )    
                raise RuntimeError, errorMessage     
    
        
    def getDataInput( self ):
        """
        Returns the Plugin Input Data
        """
        EDVerbose.DEBUG( "*** EDPlugin.getDataInput")
        return self.m_xsDataInput
    
    
    def setDataOutput( self, _xsDataOutput ):
        """
        Sets the Plugin Output Data
        """
        EDVerbose.DEBUG( "*** EDPlugin.setDataOutput: " + EDString ( _xsDataOutput ))
        self.m_xsDataOutput = _xsDataOutput
        
        
    def getDataOutput( self ):
        """
        Returns the Plugin Output Data
        """
        EDVerbose.DEBUG( "*** EDPlugin.getDataOutput")
        return self.m_xsDataOutput

    
    def exportDataOutput( self, _edPlugin = None ):
        """
        Deprecated
        Exports the Plugin Output Data to slot
        """
        EDVerbose.DEBUG( "*** EDPlugin.exportDataOutput")
        self.m_edSlotExportDataOutput.call( self.m_xsDataOutput )


    def connectExportDataOutput( self, _oMethod ):
        """
        Deprecated
        """
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_edSlotExportDataOutput.connect( _oMethod )
        self.synchronizeOff()
        
        
    def generateExecutiveSummary( self, _edPlugin ):
        """
        This method, which should be implemented by sub-classes, generates an executive summary (user-related output summary).
        """
        EDVerbose.DEBUG( "*** EDPlugin.generateExecutiveSummary" )

    
    def setExecutionContext(self, _edStringExecutionContext):
        """
        Internal
        Different context is defined whether the plugin is executed via an application or a test framework
        Used for configuration retrieval purpose
        """
        self.m_edStringExecutionContext = _edStringExecutionContext


    def addErrorMessage( self,  _edStringErrorMessage ):
        """
        Adds an error message to the error messages list
        """
        self.m_edListErrorMessages.add( EDString( _edStringErrorMessage ) )
        
        
    def getErrorMessages( self ):
        """
        Returns the error messages list
        """
        return self.m_edListErrorMessages
    
    
    def addWarningMessage( self, _edStringWarningMessage ):
        """
        Adds a warning message to the warning messages list
        """
        EDVerbose.DEBUG( "*** EDPlugin.addWarningMessage : " + _edStringWarningMessage )
        self.m_edListWarningMessages.add( EDString( _edStringWarningMessage ) )
        
        
    def getWarningMessages( self ):
        """
        Returns the warning messages list
        """
        return self.m_edListWarningMessages
    
    
    def writeDataInput( self, _edObject ):
        """
        Writes the input data object into a working dir xml file 
        """
        EDVerbose.DEBUG( "*** EDPlugin.writeDataInput")
        xsDataInput = self.getDataInput()
        if ( xsDataInput is not None ):
            edStringPathDataInput = EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.getPluginName() + "_dataInput.xml" )
            EDUtilsFile.writeFile( edStringPathDataInput,  xsDataInput.marshal() )
        
        
    def writeDataOutput( self, _edObject ):
        """
        Writes the output data object into a working dir xml file 
        """
        EDVerbose.DEBUG( "*** EDPlugin.writeDataOutput")
        xsDataOutput = self.getDataOutput()
        if ( xsDataOutput is not None ):
            edStringPathDataOutput = EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.getPluginName() + "_dataOutput.xml" )
            EDUtilsFile.writeFile( edStringPathDataOutput,  xsDataOutput.marshal() )
            
            
    def getBaseName( self ):
        """
        Returns the plugin base name
        """
        if ( self.m_edStringBaseName is None ):
            self.m_edStringBaseName = self.createBaseName()
        return EDString( self.m_edStringBaseName )
    
    
    def setBaseName( self, _edStringBaseName ):
        """
        Sets the plugin base name
        """
        self.synchronizeOn()
        self.m_edStringBaseName  = EDString( _edStringBaseName )
        self.setName( self.m_edStringBaseName )
        self.synchronizeOff()
            
            
    def createBaseName( self ):
        """
        Generates the plugin base name: (<date>-<random number>-<base name>)
        """
        # Create the base name from the class name + date
#        edStringBaseName  = self.getClassName()
#        edStringBaseName += "-" + EDTime.getTimeName()
#        edStringBaseName += "-%04i" % EDRandom.randInt( 0,9999 )
        edStringBaseName  = EDTime.getTimeName()
        edStringBaseName += "-%04i" % EDRandom.randInt( 0,9999 )
        edStringBaseName += "-%s" % self.getClassName()
        return edStringBaseName
        

    def setBaseDirectory( self, _edStringBaseDirectory ):
        """
        Sets the plugin base directory
        """
        EDVerbose.DEBUG( "*** EDPlugin.setBaseDirectory : " + _edStringBaseDirectory )
        #self.synchronizeOn()
        self.m_edStringBaseDirectory = EDString( _edStringBaseDirectory )
        #self.synchronizeOff()
        

    def getBaseDirectory( self ):
        """
        Returns the plugin base directory
        """
        EDVerbose.DEBUG( "*** EDPlugin.getBaseDirectory : %s" % self.m_edStringBaseDirectory)
        returnValue = None
        if ( self.m_edStringBaseDirectory is not None ):
            returnValue = EDString( self.m_edStringBaseDirectory )
        return returnValue


    def setWorkingDirectory( self, _edStringWorkingDirectory ):
        """
        Sets the plugin working directory
        """
        EDVerbose.DEBUG( "*** EDPlugin.setWorkingDirectory : " + _edStringWorkingDirectory )
        #self.synchronizeOn()
        self.m_edStringWorkingDirectory = EDString( _edStringWorkingDirectory )
        EDDiskExplorer.createFolder( self.m_edStringWorkingDirectory )
        #self.synchronizeOff()
        

    def getWorkingDirectory( self ):
        """
        Returns the plugin base directory
        """
        EDVerbose.DEBUG( "*** EDPlugin.getWorkingDirectory : %s" % self.m_edStringWorkingDirectory )
        returnValue = None
        if ( self.m_edStringWorkingDirectory is not None ):
            returnValue = EDString( self.m_edStringWorkingDirectory )
        return returnValue
    
    
    def checkMandatoryParameters( self, _xsData, _strParamName ):
        """
        Checks that a mandatory parameter exists in the data
        If not, an error message is added in the list and the plugin fails
        """            
            
        if( _xsData == None ):
            errorMessage = EDMessage.ERROR_MANDATORY_PARAM_MISSING_02 % ( self.getPluginName() + '.checkMandatoryParameters', _strParamName )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )    
            raise RuntimeError, errorMessage 
        
        
    def checkImportantParameters( self, _xsData, _strParamName ):
        """
        Checks that a specific parameter exists in the data
        If not, a warning message is added in the list
        """
        if( _xsData == None ):
            warningMessage = EDMessage.WARNING_IMPORTANT_PARAM_MISSING_02 % ( self.getPluginName() + '.checkImportantParameters', _strParamName )
            EDVerbose.warning( warningMessage )
            self.addWarningMessage( warningMessage )  
            
    
    def addExecutiveSummaryLine( self, _edStringExecutiveSummaryLine ):
        """
        Add a line to the executive summary string.
        """
        EDVerbose.DEBUG( "*** EDPlugin.addExecutiveSummaryLine : %r" % _edStringExecutiveSummaryLine )
        edStringExecutiveSummaryLine = _edStringExecutiveSummaryLine
        if ( not EDString( edStringExecutiveSummaryLine ).equals( EDString.empty() ) ):
             if ( EDString( edStringExecutiveSummaryLine[-1] ).equals( "\n" ) ):
                 edStringExecutiveSummaryLine = edStringExecutiveSummaryLine[:-1]
        self.m_edListExecutiveSummaryLines.add( EDString( edStringExecutiveSummaryLine ) )


    def addExecutiveSummarySeparator( self, _edStringSeparator=None ):
        """
        Adds a separator to split the executive summary into different parts
        Default is a dotted line
        """
        edStringSeparator = _edStringSeparator
        if ( edStringSeparator is None ):
            edStringSeparator = self.m_edStringExecutiveSummarySeparator
        # Check that the last line doesn't already contain a separator
        if ( not EDString( self.m_edListExecutiveSummaryLines[-1] ).equals( edStringSeparator ) ):
            self.addExecutiveSummaryLine( edStringSeparator )
            
        
    def getListExecutiveSummaryLines( self ):
        """
        Returns the executive summary (list of text lines)
        """
        return EDList( self.m_edListExecutiveSummaryLines ) 


    def verboseScreenExecutiveSummary( self ):
        """
        Prints the executive summary on the screen
        """
        for line in self.getListExecutiveSummaryLines():
            EDVerbose.screen( line )

    def verboseDebug( self, _edStringMessage ):
        EDVerbose.DEBUG( self.getPluginName() + " : " + _edStringMessage)