#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControl.py 708 2009-05-13 13:20:02Z incardon $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDList
from EDImportLib import EDVerbose
from EDImportLib import EDString
 
from EDPlugin      import EDPlugin
from EDMessage     import EDMessage
from EDApplication import EDApplication


class EDPluginControl( EDPlugin ):
    """
    An EDPluginControl is a plugin that is responsible for a EDPluginExec or EDPluginControl plugin execution:
    It is responsible for:
    - The EDPluginExec or EDPluginControl Workflow
    - The data propagation between the EDPluginExec
    - The translation between generic and specific data models via EDHandler classes
    - The error/warning propagation
    - The executive summaries propagation
    """
    
    def __init__ ( self ):
        """
        """
        EDPlugin.__init__( self )
        self.m_edStringPluginToBeControlledName = None
            
    
    def loadPlugin( self, _edStringPluginToBeControlledName = None, _edStringBaseName = None ): 
        """
        This method loads and returns a reference to the plugin to be controlled.
        
        The name of the plugin to be controlled can either be passed as an
        argument, or bet set before calling this method using the 
        "setPluginToBeControlledName". 
        
        The base name of the plugin to be controlled is used as the working
        directory name of the plugin in question. If no argument is supplied
        the name of the plugin is used as base name. In the case of creation of
        several plugins to be launched simultaneously, the base name should be
        different for each plugin and hence must be provided explicitly.
        """
        EDVerbose.DEBUG( "*** EDPluginControl.loadPlugin" )
        if ( _edStringPluginToBeControlledName is None ):
            edStringPluginName = self.m_edStringPluginToBeControlledName
        else:
            edStringPluginName = _edStringPluginToBeControlledName
        edPlugin = EDApplication.loadPlugin( edStringPluginName )
        if ( edPlugin is None ):
            errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ( "EDPluginControl.loadPlugin", edStringPluginName )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )              
            raise RuntimeError, errorMessage
        edPlugin.setBaseDirectory( self.getWorkingDirectory() )
        if ( _edStringBaseName is None ):
            edPlugin.setBaseName( edStringPluginName )
        else:
            edPlugin.setBaseName( _edStringBaseName )            
        return edPlugin
    
            
    def setPluginToBeControlledName( self, _edStringPluginToBeControlledName ):
        """
        Sets the name of the plugin to be controlled.
        """
        self.m_edStringPluginToBeControlledName = EDString( _edStringPluginToBeControlledName )


    def getPluginToBeControlledName( self ):
        """
        Returns the name of the plugin to be controlled.
        """
        return self.m_edStringPluginToBeControlledName


    def addWarningMessages( self, _edListWarningMessages ):
        """
        Adds a list of warning messages in the existing list of warning messages
        """
        EDVerbose.DEBUG( "*** EDPluginControl.addWarningMessages")
        self.synchronizeOn()
        for edStringWarningMessage in _edListWarningMessages:
            self.addWarningMessage( edStringWarningMessage )
        self.synchronizeOff()
        
        
    def addErrorMessages( self, _edListErrorMessages ):
        """
        Adds a list of error messages in the existing list of error messages
        """
        EDVerbose.DEBUG( "*** EDPluginControl.addErrorMessages")
        self.synchronizeOn()
        for edStringErrorMessage in _edListErrorMessages:
            self.addErrorMessage( edStringErrorMessage )
        self.synchronizeOff()
        
        
    def retrieveFailureMessages( self, _edPlugin, _edStringMethodCaller ):
        """
        Propagates failure messages from a plugin including unexpected errors
        Should be called in the plugin control method invoked when a plugin exec fails (doActionFailure<>)
        """
        EDVerbose.DEBUG( "*** EDPluginControl.retrieveFailureMessages")
        self.retrieveWarningMessages( _edPlugin )
        self.retrieveErrorMessages( _edPlugin, _edStringMethodCaller, True )
        
        
    def retrieveSuccessMessages( self, _edPlugin, _edStringMethodCaller ):
        """
        Propagates success messages from a plugin
        Error messages are retrieved because a plugin could end successfully with errors (depending on the use case)
        In this case, there is no check for unexpected errors
        """
        EDVerbose.DEBUG( "*** EDPluginControl.retrieveSuccessMessages")
        self.retrieveWarningMessages( _edPlugin )
        self.retrieveErrorMessages( _edPlugin, _edStringMethodCaller, False )
        
        
    def retrieveErrorMessages( self, _edPlugin, _edStringMethodCaller, _bFailure ):
        """
        Propagates error messages from a plugin
        if _bFailure is true, this method has been called from retrieveFailureMessages
        in this case, checks for potential unexpected errors coming from the EDPluginExec
        """
        EDVerbose.DEBUG( "*** EDPluginControl.retrieveErrorMessages")
        errorMessages = _edPlugin.getErrorMessages()
        if(( errorMessages.getNumberObjects() == 0 ) and ( _bFailure is True)):
            EDVerbose.DEBUG( "************** " + _edStringMethodCaller + ": Adding Unexpected error" )
            errorMessages.add( EDMessage.ERROR_UNEXPECTED_01 % ( _edStringMethodCaller ) )
        self.addErrorMessages( errorMessages )
            
        
    def retrieveWarningMessages( self, _edPlugin ):
        """
        Propagates warning messages from a plugin
        """
        EDVerbose.DEBUG( "*** EDPluginControl.retrieveWarningMessages")
        self.addWarningMessages( _edPlugin.getWarningMessages() )
        

    def appendExecutiveSummary( self, _edPlugin, _edStringPrefix="", _bAddSeparator=True ):
        """
        Appends the executive summary from a plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControl.appendExecutiveSummary" )
        if ( _bAddSeparator ):
            self.addExecutiveSummarySeparator()
        for line in _edPlugin.getListExecutiveSummaryLines():
            edStringLine = EDString( line )
            if ( edStringLine.equals( self.m_edStringExecutiveSummarySeparator ) ):
                 edStringLine = edStringLine[ :-len( _edStringPrefix ) ]
            self.addExecutiveSummaryLine( _edStringPrefix + edStringLine )


    def addErrorWarningMessagesToExecutiveSummary( self, _edStringErrorMessage="Error messages:", _edStringWarningMessage= "Warning messages:" ):
        """
        Adds error and warning messages (if any) in the executive summary
        """
        if ( self.getErrorMessages().getNumberObjects() != 0 ):
            self.addExecutiveSummarySeparator()
            self.addExecutiveSummaryLine( _edStringErrorMessage )
            for errorMessage in self.getErrorMessages():
                self.addExecutiveSummaryLine( errorMessage )
            self.addExecutiveSummarySeparator()
        if ( self.getWarningMessages().getNumberObjects() != 0 ):
            self.addExecutiveSummarySeparator()
            self.addExecutiveSummaryLine( _edStringWarningMessage )
            for warningMessage in self.getWarningMessages():
                self.addExecutiveSummaryLine( warningMessage )
            self.addExecutiveSummarySeparator()
    
