#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginExecProcess.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#

from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDDiskExplorer
from EDImportLib import EDActionProcess

from EDPluginExec     import EDPluginExec
from EDConfiguration  import EDConfiguration
from EDMessage        import EDMessage

from XSDataCommon import XSPluginItem

class EDPluginExecProcess( EDPluginExec ):
    """
    The super class for all plugins that execute a process
    This class manages the process to be launched:
    - Process time out management (configurable, Default: 100s)
    - Process executable to be invoked (configurable, Default 'cat')
    - Process command line to be launched (Default: empty)
    """
    
    def __init__ ( self ):
        """
        Initializes process related attributes described above
        """
        EDPluginExec.__init__( self )
        self.m_edStringCONF_EXEC_PROCESS_TIME_OUT           = "execProcessTimeOut"
        self.m_edStringConfigTimeOut                     = EDString( "100" )
        self.m_edStringCONF_EXEC_PROCESS_EXECUTABLE         = "execProcessExecutable"
        self.m_edStringConfigExecutable                  = EDString( "cat" )
        self.m_edStringConfigCommandline                 = ""

        
    def preProcess( self, _edObject = None ):
        """
        Nothing particular
        Should be cleaned up.
        """
        EDPluginExec.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.preProcess")

        
    def process( self, _edObject = None ):
        """
        Sets the process up with the executable, command line and time out
        Launches the process, in case of error, an error message is added to the list, and the plugins fails 
        """
        EDPluginExec.process( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.process starting" )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.process executing: " + self.m_edStringConfigExecutable )
        self.synchronizeOn()
        edStringCommand = EDString( self.m_edStringConfigExecutable + " " + self.m_edStringConfigCommandline ) 
        edActionProcess = EDActionProcess( edStringCommand ) 
        edActionProcess.setTimeOut( self.m_edStringConfigTimeOut )
        edActionProcess.setWorkingFolder( self.getWorkingDirectory() )
        try:
            EDVerbose.screen( self.getBaseName() + ": Processing" )
            edActionProcess.executeSynchronous()            
        except Exception, detail:
            EDVerbose.DEBUG( "*** EDPluginExecProcess.process ========================================= ERROR! ================" )
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcess.process', self.getClassName(), "cannot execute " + edStringCommand + " : " + EDString( detail ) ) 
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage 
        EDVerbose.DEBUG( "*** EDPluginExecProcess.process finished " )
        self.synchronizeOff()
       
    
    def postProcess( self, _edObject = None ):
        """
        Nothing particular
        Should be cleaned up.
        """
        EDPluginExec.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.postProcess")
        
        
    def configure( self ):
        """
        Configures the plugin with time out and executable from configuration file
        """
        EDPluginExec.configure( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.configure")
        xsPluginItem = self.getConfiguration()
        if ( xsPluginItem == None ):
            EDVerbose.warning( "*** EDPluginExecProcess.configure: No plugin item defined." )
            xsPluginItem = XSPluginItem()
        edStringTimeOut = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStringCONF_EXEC_PROCESS_TIME_OUT )
        if( edStringTimeOut == None ):
            EDVerbose.DEBUG( "*** EDPluginExecProcess.configure: No configuration parameter found for: " + self.m_edStringCONF_EXEC_PROCESS_TIME_OUT + ", using default value:" +  self.getTimeOut())
        else:
            self.setTimeOut( edStringTimeOut )
        edStringExecutable = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStringCONF_EXEC_PROCESS_EXECUTABLE )
        if( edStringExecutable == None ):
            pass
            #EDVerbose.DEBUG( "*** EDPluginExecProcess.configure: No configuration parameter found for: " + self.m_edStringCONF_EXEC_PROCESS_EXECUTABLE + ", using default value:" + self.getExecutable() )
        else:
            self.setExecutable( edStringExecutable )
        
        


    def setExecutable( self, _edStringExecutable ):
        """
        Sets the executable
        """
        self.synchronizeOn()
        self.m_edStringConfigExecutable = EDString( _edStringExecutable )
        self.synchronizeOff()
        

    def getExecutable( self ):
        """
        Sets the executable
        """
        return EDString( self.m_edStringConfigExecutable )

    
    def setCommandline( self, _edStringCommandline ):
        """
        Sets the command line
        """
        self.synchronizeOn()
        self.m_edStringConfigCommandline = EDString( _edStringCommandline )
        self.synchronizeOff()
        

    def getCommandline( self ):
        """
        Returns the command line
        """
        return EDString( self.m_edStringConfigCommandline )


    def setTimeOut( self, _edStringTimeOut ):
        """
        Sets the time out
        """
        self.synchronizeOn()
        self.m_edStringConfigTimeOut = EDString( _edStringTimeOut )
        #self.m_edStringConfigTimeOut = float( _edStringTimeOut )
        self.synchronizeOff()
        

    def getTimeOut( self ):
        """
        Returns the time out
        """
        return EDString( self.m_edStringConfigTimeOut )
        #return float( self.m_edStringConfigTimeOut )
