#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginExecProcessScript.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList
from EDImportLib import EDStreamFile
from EDImportLib import EDDiskExplorer
from EDImportLib import EDTime
from EDImportLib import EDRandom
from EDImportLib import EDTime

from EDMessage                    import EDMessage
from EDUtilsFile                  import EDUtilsFile
from EDPluginExecProcess          import EDPluginExecProcess
from EDActionExecuteSystemCommand import EDActionExecuteSystemCommand
from EDConfiguration              import EDConfiguration 

from XSDataCommon import XSPluginItem

from EDImportSystem import PyOs


class EDPluginExecProcessScript( EDPluginExecProcess ):
    """
    The super class for all EDNA plugins that execute a task using a script.
    This class manages the process script to be executed:
    - Configuration options:
      * execProcessScriptShell         : The shell that will execute the script, written into the top line of the script,
                                         for example : #!/bin/bash
      * execProcessScriptExecutor      : If defined, this command is used for launching the shell. It can be used for
                                         example to launch the shell using a grid engine. (Note that the 
                                         execProcessScriptExecutor will be replaced by execProcessExecutable)
      * execProcessScriptExecutable    : The path to the third-party executable that will be launched in the script.
      * execProcessScriptSetupCCP4     : Path to CCP4 setup file (if required)
      * execProcessScriptVersionString : A string which describes the version of the execProcessScriptExecutable,
                                         which should match the version string written into the program log,
                                         for example: "Version 7.0.1  for Image plate and CCD data 20th August 2007"
                                         (Note that this configuration option will be moved to EDPluginExecProcess)
    - The list of compatible 3rd party executable version the plugin should support (should be move up to EDPluginExecProcess)
    - The list of pre-execution commands
    - The script command line that invokes the executable, which is written into the script
      (the EDPluginExecProcess command line is not used, but will be in the future: It will consist of the
      execProcessExecutable and the path to the script file.)
    - The list of execution commands for the execProcessScriptExecutable
    - The list of post-execution commands
    - The script base name (<date>-<random number>-<base name>)
    - The script file name
    - The script file path
    - The standard output log file name
    - The error output file log file name
    - The poll sleep time for checking if the process has finished (set to 1 s).
    """
    
    def __init__ ( self ):
        """
        Initializes process related attributes described above
        """
        EDPluginExecProcess.__init__( self )
        self.m_edStringCONF_EXEC_PROCESS_SCRIPT_SHELL           = "execProcessScriptShell"
        self.m_edStringConfigShell                           = EDString( "/bin/bash" )
        self.m_edStringCONF_EXEC_PROCESS_SCRIPT_EXECUTOR        = "execProcessScriptExecutor"
        self.m_edStringScriptExecutor                        = None
        self.m_edStringCONF_EXEC_PROCESS_SCRIPT_EXECUTABLE      = "execProcessScriptExecutable"
        self.m_edStringScriptExecutable                      = None
        self.m_edStringCONF_EXEC_PROCESS_SCRIPT_SETUP_CCP4      = "execProcessScriptSetupCCP4"
        self.m_edStringConfigSetupCCP4                       = None
        self.m_edStringCONF_EXEC_PROCESS_SCRIPT_VERSION_STRING  = "execProcessScriptVersionString"
        self.m_edStringVersion                               = None
        self.m_edListCompatibleVersions                      = EDList()
        self.m_edStringScriptCommandline          = EDString()
        self.m_edStringScriptBaseName             = None
        self.m_edStringScriptLogFileName          = None
        self.m_edStringScriptErrorLogFileName     = None
        self.m_edStringScriptFileName             = None
        self.m_bRequireCCP4                       = False
        self.m_edListCommandPreExecution          = EDList()
        self.m_edListCommandExecution             = EDList()
        self.m_edListCommandPostExecution         = EDList()
        self.m_iPollScriptProcessTime              = 1 # [s]
        
        
    def preProcess( self, _edObject = None ):
        """
        Calls the parent preProcess method
        Generates the script
        """
        EDPluginExecProcess.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.preProcess")
        # The generateScript method will be called at the end of the preProcess method
        self.connectPreProcess( self.generateScript )            
        
        
    def generateScript( self, _edObject = None ):
        """
        This method prepares the script and writes it to disk.
        """
        edStringScript = self.prepareScript()
        self.writeExecutableScript( edStringScript )
            
        
    def process( self, _edObject = None ):
        """
        This method starts the execution of the EDNA script using EDActionExecuteSystemCommand
        In case of failure, an error message is added to the list and the plugin fails.
        
        (Note that normally this processing should take place in the EDPluginExecProcess.process
        method, however due to a bug in the Python subprocess handling which is used by the
        AALib ALActionProcess, see bug #61.) 
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.process starting" )
        self.synchronizeOn()
        edStringCommand = EDString( self.getExecutable() + " " + self.getCommandline() ) 
        EDVerbose.screen( self.getBaseName() + ": Processing" )
        edActionExecuteSystemCommand = EDActionExecuteSystemCommand( edStringCommand )
        edActionExecuteSystemCommand.execute()
        edTime = EDTime()
        edTime.initTime()
        bContinue = True
        bTimeOut = False
        while ( bContinue ):
            EDVerbose.DEBUG( "*** EDPluginExecProcess.process executing: " + edStringCommand )
            EDTime.sleep( self.m_iPollScriptProcessTime )
            if ( edActionExecuteSystemCommand.isEnded() ):
                EDVerbose.DEBUG( "*** EDPluginExecProcess.process - execution ended" )
                bContinue = False
            elif ( edTime.getDuration() > self.getTimeOut().atoi() ):
                EDVerbose.DEBUG( "*** EDPluginExecProcess.process - execution time out" )
                bContinue = False
                bTimeOut = True
                edActionExecuteSystemCommand.abort()
        if ( bTimeOut == True ):
            EDVerbose.DEBUG( "*** EDPluginExecProcess.process ========================================= ERROR! ================" )
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcess.process', self.getClassName(), "execution timed out!" ) 
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage 
        else:
            edStringExecutionStatus = EDString( edActionExecuteSystemCommand.getExecutionStatus() )
            # Report an error if the result is not "0" and the result is not an empty string
            if ( not edStringExecutionStatus.equals( '0' ) and not edStringExecutionStatus.equals( EDString.empty() ) ):
                EDVerbose.DEBUG( "*** EDPluginExecProcess.process ========================================= ERROR! ================" )
                edStringErrorLog = self.readProcessErrorLogFile()
                if ( edStringErrorLog is None ):
                    errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcess.process', self.getClassName(), "Execution status : " + edStringExecutionStatus ) 
                    EDVerbose.error( errorMessage )
                    self.addErrorMessage( errorMessage )
                else:
                    errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcess.process', self.getClassName(), EDString( edStringErrorLog ) ) 
                    EDVerbose.error( errorMessage )
                    self.addErrorMessage( errorMessage )
        EDVerbose.DEBUG( "*** EDPluginExecProcess.process finished " )
        self.synchronizeOff()
        
        
    def postProcess( self, _edObject = None ):
        """
        Calls the parent preProcess method
        Checks that the installed 3rd party software is in the list of compatible versions
        """
        EDPluginExecProcess.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.postProcess")
        
        # Tests the compatibles executable versions
        edListCompatibleVersions = self.getListOfCompatibleVersions()
        
        if( edListCompatibleVersions.getNumberObjects() != 0 ):
        
            bFound = False
            for compatibleVersion in edListCompatibleVersions:
                bFound = self.findStringInLog( compatibleVersion )
                if( bFound == True ):
                    break
            
            if( bFound == False ):
                errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcessScript.postProcess', self.getClassName(), "Plugin not compatible with %s, compatible versions are: %s" % (self.getStringVersion(), self.getCompatibleVersionsStringLine() ))
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
                raise RuntimeError, errorMessage


    def configure( self ):
        """
        Configures the plugin from the configuration file with the following parameters
        - The shell that will execute the script
        - Script executor
        - Script executable to be invoked
        - Path to CCP4 setup file if required
        - The 3rd party executable installed version
        """
        EDPluginExecProcess.configure( self )
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.configure")
        xsPluginItem = self.getConfiguration()
        if ( xsPluginItem == None ):
            EDVerbose.warning( "EDPluginExecProcessScript.configure: No plugin item defined." )
            xsPluginItem = XSPluginItem()
        edStringShell = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStringCONF_EXEC_PROCESS_SCRIPT_SHELL )
        if( edStringShell == None ):
            EDVerbose.DEBUG( "EDPluginExecProcessScript.configure: No configuration parameter found for: " + self.m_edStringCONF_EXEC_PROCESS_SCRIPT_SHELL + ", using default value: " + self.getScriptShell() )
        else:
            self.setScriptShell( edStringShell )
        edStringScriptExecutor = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStringCONF_EXEC_PROCESS_SCRIPT_EXECUTOR )
        if( edStringScriptExecutor == None ):
            self.setScriptExecutor( self.getScriptShell() )
            EDVerbose.DEBUG( "EDPluginExecProcessScript.configure: No configuration parameter found for: " + self.m_edStringCONF_EXEC_PROCESS_SCRIPT_EXECUTOR + ", using script shell: " + self.getScriptShell() )
        else:
            self.setScriptExecutor( edStringScriptExecutor )
        if ( self.getScriptExecutable() is None ):
            edStringScriptExecutable = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStringCONF_EXEC_PROCESS_SCRIPT_EXECUTABLE )
            if( edStringScriptExecutable == None ):
                errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcessScript.process', self.getClassName(), "Configuration parameter missing: "+self.m_edStringCONF_EXEC_PROCESS_SCRIPT_EXECUTABLE ) 
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
                raise RuntimeError, errorMessage             
            else:
                # Check that the executable file exists
                if ( EDDiskExplorer.isFile( edStringScriptExecutable ) == False ):
                    errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginExecProcessScript.process', self.getClassName(), "Cannot find configured " + self.m_edStringCONF_EXEC_PROCESS_SCRIPT_EXECUTABLE + " : "+ edStringScriptExecutable ) 
                    EDVerbose.error( errorMessage )
                    self.addErrorMessage( errorMessage )
                    raise RuntimeError, errorMessage             
                else:
                    self.setScriptExecutable( edStringScriptExecutable )        
        edStringConfigSetupCCP4 = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStringCONF_EXEC_PROCESS_SCRIPT_SETUP_CCP4 )
        if( edStringConfigSetupCCP4 == None ):
            EDVerbose.DEBUG( "EDPluginExecProcessScript.configure: No configuration parameter found for: " + self.m_edStringCONF_EXEC_PROCESS_SCRIPT_SETUP_CCP4 + ", NO default value!" )
        else:
            self.setSetupCCP4( edStringConfigSetupCCP4 )        
        edStringVersion = EDConfiguration.getStringParamValue( xsPluginItem, self.m_edStringCONF_EXEC_PROCESS_SCRIPT_VERSION_STRING )
        if( edStringVersion == None ):
            EDVerbose.DEBUG( "EDPluginExecProcessScript.configure: No configuration parameter found for: " + self.m_edStringCONF_EXEC_PROCESS_SCRIPT_VERSION_STRING + ", NO default value!" )
        else:
            self.setStringVersion( edStringVersion )        
        if ( self.m_edStringScriptBaseName == None ):
            self.setScriptBaseName( self.getBaseName() )
        if ( self.m_edStringScriptFileName == None ):
            self.setScriptFileName( self.m_edStringScriptBaseName + ".sh" )
        if ( self.m_edStringScriptLogFileName == None ):
            self.setScriptLogFileName( self.m_edStringScriptBaseName + ".log" )
        if ( self.m_edStringScriptErrorLogFileName == None ):
            self.setScriptErrorLogFileName( self.m_edStringScriptBaseName + ".err" )
            
            
    def getListOfCompatibleVersions( self ):
        """
        Returns the list of compatible executable versions the plugin supports
        """
        return self.m_edListCompatibleVersions
    
    
    def addCompatibleVersion( self, _edStringCompatibleVersion ):
        """
        Adds a compatible executable version to the list
        """
        self.m_edListCompatibleVersions.add( _edStringCompatibleVersion )
    
    
    def getCompatibleVersionsStringLine( self ):
        """
        This Method constructs a string line by concatening the compatible versions this plugin supports
        This is for Log message purpose only.
        """
        edStringCompatibleVersionsStringLine = ""
        for compatibleVersion in self.getListOfCompatibleVersions():
            edStringCompatibleVersionsStringLine = edStringCompatibleVersionsStringLine + compatibleVersion + " ; " 
            
        return edStringCompatibleVersionsStringLine

 
    def setRequireCCP4( self, _bReqireCCP4 ):
        """
        Sets a requirement on CCP4
        """
        self.synchronizeOn()
        self.m_bRequireCCP4 = _bReqireCCP4
        self.synchronizeOff()


    def getRequireCCP4( self ):
        """
        Returns True if the executable requires CCP4
        """
        return self.m_bRequireCCP4

        
    def setListCommandExecution( self, _edListCommandExecution ):
        """
        Sets the list of execution commands
        """
        self.synchronizeOn()
        if ( _edListCommandExecution is not None ):
            self.m_edListCommandExecution = EDList( _edListCommandExecution )
        self.synchronizeOff()
        

    def addListCommandExecution( self, _edStringCommandExecution ):
        """
        Adds an execution command to the list
        """
        self.synchronizeOn()
        if ( _edStringCommandExecution is not None ):
            self.m_edListCommandExecution.add( EDString( _edStringCommandExecution ) )
        self.synchronizeOff()

    
    def getListCommandExecution( self ):
        """
        Returns the list of execution commands
        """
        self.synchronizeOn()
        edObject = None
        if ( self.m_edListCommandExecution is not None ):
            edObject = EDList( self.m_edListCommandExecution )
        self.synchronizeOff()
        return edObject

    
    def setListCommandPreExecution( self, _edListCommandPreExecution ):
        """
        Sets the list of pre execution commands
        """
        self.synchronizeOn()
        if ( _edListCommandPreExecution is not None ):
            self.m_edListCommandPreExecution = EDList( _edListCommandPreExecution )
        self.synchronizeOff()
        

    def addListCommandPreExecution( self, _edStringCommandPreExecution ):
        """
        Adds a pre execution command to the list
        """
        self.synchronizeOn()
        if ( _edStringCommandPreExecution is not None ):
            self.m_edListCommandPreExecution.add( EDString( _edStringCommandPreExecution ) )
        self.synchronizeOff()

    
    def getListCommandPreExecution( self ):
        """
        Returns the list of pre execution commands
        """
        self.synchronizeOn()
        edObject = None
        if ( self.m_edListCommandPreExecution != None ):
            edObject = EDList( self.m_edListCommandPreExecution )
        self.synchronizeOff()
        return edObject

    
    def setListCommandPostExecution( self, _edListCommandPostExecution ):
        """
        Sets the list of post execution commands
        """
        self.synchronizeOn()
        if ( _edListCommandPostExecution is not None ):
            self.m_edListCommandPostExecution = EDList( _edListCommandPostExecution )
        self.synchronizeOff()
        

    def addListCommandPostExecution( self, _edStringCommandPostExecution ):
        """
        Adds a post execution command to the list
        """
        self.synchronizeOn()
        if ( _edStringCommandPostExecution is not None ):
            self.m_edListCommandPostExecution.add( EDString( _edStringCommandPostExecution ) )
        self.synchronizeOff()

    
    def getListCommandPostExecution( self ):
        """
        Returns the list of post execution commands
        """
        self.synchronizeOn()
        edObject = None
        if ( self.m_edListCommandPostExecution != None ):
            edObject = EDList( self.m_edListCommandPostExecution )
        self.synchronizeOff()
        return edObject

    
    def setScriptShell( self, _edStringScriptShell ):
        """
        Sets the script shell
        """
        self.synchronizeOn()
        self.m_edStringConfigShell = EDString( _edStringScriptShell )
        self.synchronizeOff()
        

    def getScriptShell( self ):
        """
        Returns the script shell
        """
        return EDString( self.m_edStringConfigShell )

    
    def setScriptExecutor( self, _edStringScriptExecutor ):
        """
        Sets the script executor
        """
        self.synchronizeOn()
        self.m_edStringScriptExecutor = EDString( _edStringScriptExecutor )
        self.synchronizeOff()
        

    def getScriptExecutor( self ):
        """
        Returns the script executor
        """
        return EDString( self.m_edStringScriptExecutor )

    
    def setSetupCCP4( self, _edStringSetupCCP4 ):
        """
        Sets the path to CCP4 setup file
        """
        self.synchronizeOn()
        self.m_edStringConfigSetupCCP4  = EDString( _edStringSetupCCP4 )
        self.synchronizeOff()
        

    def getSetupCCP4( self ):
        """
        Returns the path to CCP4 setup file
        """
        return EDString( self.m_edStringConfigSetupCCP4 )

    
    def setScriptBaseName( self, _edStringScriptBaseName ):
        """
        Sets the script name
        """
        self.synchronizeOn()
        self.m_edStringScriptBaseName  = EDString( _edStringScriptBaseName )
        self.synchronizeOff()
        

    def getScriptBaseName( self ):
        """
        Returns the script name 
        if None, create it: <date>-<random number>-<base name>
        """
        if ( self.m_edStringScriptBaseName is None ):
            self.m_edStringScriptBaseName = self.createBaseName()
        return EDString( self.m_edStringScriptBaseName )

    
    def setScriptLogFileName( self, _edStringScriptLogFileName ):
        """
        Sets the standard output log file name
        """
        self.synchronizeOn()
        self.m_edStringScriptLogFileName  = EDString( _edStringScriptLogFileName )
        self.synchronizeOff()
        

    def setScriptErrorLogFileName( self, _edStringScriptErrorLogFileName ):
        """
        Sets the error output log file name
        """
        self.synchronizeOn()
        self.m_edStringScriptErrorLogFileName  = EDString( _edStringScriptErrorLogFileName )
        self.synchronizeOff()

    
    def getScriptLogFileName( self ):
        """
        Returns the standard output log file name
        """
        return EDString( self.m_edStringScriptLogFileName )

    
    def getScriptErrorLogFileName( self ):
        """
        Returns the error output log file name
        """
        return EDString( self.m_edStringScriptErrorLogFileName )

    
    def setScriptFileName( self, _edStringScriptFileName ):
        """
        Sets the script file name
        """
        self.synchronizeOn()
        self.m_edStringScriptFileName  = EDString( _edStringScriptFileName )
        self.synchronizeOff()
        

    def getScriptFileName( self ):
        """
        Returns the script file name
        """
        return EDString( self.m_edStringScriptFileName )

    
    def getScriptFilePath( self ):
        """
        Returns the script file path
        """
        return EDString( EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.m_edStringScriptFileName ) )

    
    def setScriptExecutable( self, _edStringScriptExecutable ):
        """
        Sets the executable path
        """
        self.synchronizeOn()
        self.m_edStringScriptExecutable  = EDString( _edStringScriptExecutable )
        self.synchronizeOff()
        

    def getScriptExecutable( self ):
        """
        Returns the executable path
        """
        returnValue = self.m_edStringScriptExecutable
        if ( returnValue is not None ):
            returnValue = EDString( self.m_edStringScriptExecutable )
        return returnValue

    
    def setScriptCommandline( self, _edStringScriptCommandline ):
        """
        Sets the script command line (parameters to executable)
        """
        self.synchronizeOn()
        self.m_edStringScriptCommandline  = EDString( _edStringScriptCommandline )
        self.synchronizeOff()
        

    def getScriptCommandline( self ):
        """
        Returns the script command line (parameters to executable)
        """
        return EDString( self.m_edStringScriptCommandline )

    
    def setStringVersion( self, _edStringVersion ):
        """
        Sets the executable version
        """
        self.synchronizeOn()
        self.m_edStringVersion = EDString( _edStringVersion )
        self.synchronizeOff()
        

    def getStringVersion( self ):
        """
        Returns the executable version
        """
        returnValue = None
        if ( self.m_edStringVersion is not None ):
            returnValue = EDString( self.m_edStringVersion ) 
        return returnValue
    
        
    def prepareScript( self ):
        """
        Returns a string containing the script.
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.generateScript")
        self.synchronizeOn()
        edStringScript = EDString()
        edStringScript += EDString( "#!" + self.getScriptShell() + "\n" )
        edStringScript += EDString( "cd " + self.getWorkingDirectory() + "\n" )
        if ( self.m_bRequireCCP4 ):
            if ( self.m_edStringConfigSetupCCP4 == None or self.m_edStringConfigSetupCCP4 == EDString.empty() ):
                EDVerbose.DEBUG( "*** EDPluginExecProcessScript.generateScript : CCP4 setup script not defined." )
            else:
                edStringScript += EDString( ". " + self.m_edStringConfigSetupCCP4 + "\n" )
        # Add pre-execution commands - if any
        for edStringCommandPreExecution in self.m_edListCommandPreExecution:
            edStringScript += EDString( edStringCommandPreExecution ) + "\n"
        # Execution
        edStringScript += EDString( self.m_edStringScriptExecutable + " " + self.m_edStringScriptCommandline + " > " + self.m_edStringScriptLogFileName + " 2> " + self.m_edStringScriptErrorLogFileName )
        # Add execution commands - if any
        if ( len( self.m_edListCommandExecution ) == 0 ):
            edStringScript += EDString( "\n" )
        else:
            edStringScript += EDString( " << EOF-EDPluginExecProcessScript\n" )
            for edStringCommandExecution in self.m_edListCommandExecution:
                edStringScript += EDString( edStringCommandExecution ) + "\n"
            edStringScript += EDString( "EOF-EDPluginExecProcessScript\n" )
        # Add post-execution commands - if any
        if ( len( self.m_edListCommandPostExecution ) > 0 ):
            for edStringCommandPostExecution in self.m_edListCommandPostExecution:
                edStringScript += EDString( edStringCommandPostExecution ) + "\n"            
        self.synchronizeOff()
        return edStringScript
    

    def writeExecutableScript( self, _edStringScript ):
        """
        Writes the script to the script file.
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.writeScriptToFile")
        self.synchronizeOn()
        
        edStreamFileScript = EDStreamFile( self.getScriptFilePath() )
        edStreamFileScript.write( _edStringScript )
        edStreamFileScript.close()
        self.synchronizeOff()
        self.setExecutable( self.getScriptExecutor() )
        self.setCommandline( self.getScriptFilePath() )
        
        
    def findStringInLog( self, _edStringInput ):
        """
        Returns True if a string exists in the log file
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.findStringInLog")
        bTestSuccess = False
        edStringLogFileContent = self.readProcessLogFile()
        
        if ( EDString( edStringLogFileContent ).existSubString( _edStringInput ) == True ):
            bTestSuccess = True

        return bTestSuccess
 
            
    def testVersion( self ):
        """
        Deprecated not used
        See postProcess for version checking
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.testVersion")
        bCorrectVersion = False
        if ( self.findStringInLog( self.m_edStringVersion ) != None ):
            bCorrectVersion = True
        return bCorrectVersion
    

    def writeProcessFile( self, _edStringFileName, _edStringContent ):
        """
        Main method to write a file in the plugin working directory
        Such a file is called process file
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.writeProcessFile")
        edStringFilePath = EDDiskExplorer.mergePath( self.getWorkingDirectory(), _edStringFileName )
        edStringFileContent = EDUtilsFile.writeFile( edStringFilePath, _edStringContent )

    
    def readProcessFile( self, _edStringFileName ):
        """
        Returns the file content of a process file
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.readProcessFile")
        edStringFilePath = EDDiskExplorer.mergePath( self.getWorkingDirectory(), _edStringFileName )
        edStringFileContent = None
        if EDDiskExplorer.isFile( edStringFilePath ):
            edStringFileContent = EDUtilsFile.readFile( edStringFilePath )
        return edStringFileContent

    
    def readProcessLogFile( self ):
        """
        Returns the content of the process standard output log file
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.readProcessLogFile")
        edStringLogFileContent = self.readProcessFile( self.getScriptLogFileName() )
        return edStringLogFileContent
        

    def readProcessErrorLogFile( self ):
        """
        Returns the content of the process error output log file
        """
        EDVerbose.DEBUG( "*** EDPluginExecProcessScript.readProcessErrorLogFile")
        edStringErrorLogFileContent = self.readProcessFile( self.getScriptErrorLogFileName() )
        return edStringErrorLogFileContent
        
