#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDUtilsSymmetry.py 690 2009-05-07 15:14:03Z incardon $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDObject
from EDImportLib                         import EDString
from EDImportLib                         import EDVerbose


class EDUtilsSymmetry (EDObject):
    """
    This utility class contains methods useful for handling symmetries.
    """
    
    def getMinimumSymmetrySpaceGroupFromBravaisLattice( _edStringBravaisLattice ):
        """
        This static method returns an EDString containing the name of the space
        group with minimum symmetry that corresponds to a given two-letter Bravais
        Lattice.
        
        Note that the "hR" Bravais Lattice is translated into the "H3" and not the
        "R3" space group since the "H3" space group is the one used in MX in general
        and in particular for structures deposited in the PDB.
        
        If the Bravais Lattice is not recognised a "None" object is returned.
        """
        edStringSpaceGroup = None
        edStringBravaisLattice = EDString( _edStringBravaisLattice )
        if ( edStringBravaisLattice.equals( "aP" ) ):
            edStringSpaceGroup = EDString( "P1" )
            
        elif ( edStringBravaisLattice.equals( "mP" ) ):
            edStringSpaceGroup = EDString( "P2" )
            
        elif ( edStringBravaisLattice.equals( "mC" ) or \
               edStringBravaisLattice.equals( "mI" ) ):
            edStringSpaceGroup = EDString( "C2" )
            
        elif ( edStringBravaisLattice.equals( "oP" ) ):
            edStringSpaceGroup = EDString( "P222" )
            
        elif ( edStringBravaisLattice.equals( "oA" ) or \
               edStringBravaisLattice.equals( "oB" ) or \
               edStringBravaisLattice.equals( "oC" ) or \
               edStringBravaisLattice.equals( "oS" ) ):
            edStringSpaceGroup = EDString( "C222" )
            
        elif ( edStringBravaisLattice.equals( "oF" ) ):
            edStringSpaceGroup = EDString( "F222" )
            
        elif ( edStringBravaisLattice.equals( "oI" ) ):
            edStringSpaceGroup = EDString( "I222" )
            
        elif ( edStringBravaisLattice.equals( "tP" ) or \
               edStringBravaisLattice.equals( "tC" ) ):
            edStringSpaceGroup = EDString( "P4" )
            
        elif ( edStringBravaisLattice.equals( "tI" ) or \
               edStringBravaisLattice.equals( "tF" ) ):
            edStringSpaceGroup = EDString( "I4" )
            
        elif ( edStringBravaisLattice.equals( "hP" ) ):
            edStringSpaceGroup = EDString( "P3" )
            
        elif ( edStringBravaisLattice.equals( "hR" ) ):
            edStringSpaceGroup = EDString( "H3" )
            
        elif ( edStringBravaisLattice.equals( "cP" ) ):
            edStringSpaceGroup = EDString( "P23" )
            
        elif ( edStringBravaisLattice.equals( "cF" ) ):
            edStringSpaceGroup = EDString( "F23" )
            
        elif ( edStringBravaisLattice.equals( "cI" ) ):
            edStringSpaceGroup = EDString( "I23" )
            
        return edStringSpaceGroup
    getMinimumSymmetrySpaceGroupFromBravaisLattice = staticmethod( getMinimumSymmetrySpaceGroupFromBravaisLattice )
    
    
    def getNumberOfSymmetryOperatorsFromSpaceGroupITNumber( _edStringSpaceGroupId, _edStringSymmetryTableFileName ):
        """
        """
        return EDUtilsSymmetry.getNumberOfSymmetryOperators( _edStringSpaceGroupId, 0, _edStringSymmetryTableFileName )
    getNumberOfSymmetryOperatorsFromSpaceGroupITNumber = staticmethod( getNumberOfSymmetryOperatorsFromSpaceGroupITNumber )
 
 
    def getNumberOfSymmetryOperatorsFromSpaceGroupName( _edStringSpaceGroupName, _edStringSymmetryTableFileName ):
        """
        """
        return EDUtilsSymmetry.getNumberOfSymmetryOperators( _edStringSpaceGroupName, 3, _edStringSymmetryTableFileName )
    getNumberOfSymmetryOperatorsFromSpaceGroupName = staticmethod( getNumberOfSymmetryOperatorsFromSpaceGroupName )
    
       
    def getNumberOfSymmetryOperators( _edStringSpaceGroupIdOrName, _iIndex, _edStringSymmetryTableFileName ):
        """
        """
        edStringNumOperators = None
        xsDataNumberNumStructInUC = None
        
        try:
            pyFile = open( _edStringSymmetryTableFileName )
        except:
            EDVerbose.error( "*** EDUtilsSymmetry.getNumberOfSymmetryOperators: Problem when reading " + _edStringSymmetryTableFileName )      
            raise
        
        for pyStrLine in pyFile.xreadlines():
            pyListItems = pyStrLine.split(" ")
            if ( len( pyListItems ) > 3 and pyListItems[_iIndex] == _edStringSpaceGroupIdOrName ):
                edStringNumOperators = EDString( pyListItems[1] )
        
        return edStringNumOperators
    getNumberOfSymmetryOperators = staticmethod( getNumberOfSymmetryOperators )