#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecute.py 695 2009-05-11 09:33:20Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer
from EDImportLib                         import EDTime

from EDAssert                            import EDAssert
from EDUtilsPath                         import EDUtilsPath
from EDUtilsTest                         import EDUtilsTest
from EDTestCasePlugin                    import EDTestCasePlugin
from EDApplicationCheckPlugin            import EDApplicationCheckPlugin
from EDImportSystem                      import PyOs


class EDTestCasePluginExecute( EDTestCasePlugin ):
    """
    This is the main class for all Plugin Execution tests
    """
    
    def __init__( self, _edStringPluginName, _edStringPluginDir = None, _edStringTestName = None ):
        """
        Initialize the Plugin Execution test
        - Edna site
        - Configuration files
        - Data input file
        - Plugin launcher
        """
        EDTestCasePlugin.__init__( self, _edStringPluginName, _edStringPluginDir, _edStringTestName )
        
        self.m_edStringEdnaSite = EDUtilsPath.getEdnaSite()
        
        edStringRefConfFile = None
        if( self.m_edStringEdnaSite != None ):
            edStringRefConfFile = "XSConfiguration_" + self.getEdnaSite() + ".xml"
            
        self.m_edStringRefConfigFile = EDDiskExplorer.mergePath( EDUtilsTest.getConfigurationHome( self.getClassName() ), edStringRefConfFile )
                        
        self.m_edStringConfigurationFile = None
        self.m_edStringDataInputFile = None
        self.m_edApplicationCheckPlugin = None
        
        
    def preProcess( self ):
        """
        Initialize the plugin launcher
        """
        
        if( self.m_edStringEdnaSite == None ):
            raise RuntimeError, "EDNA_SITE must be set"
        
        self.m_edApplicationCheckPlugin = EDApplicationCheckPlugin( self.getPluginName(), self.getConfigurationFile(), self.getDataInputFile(), EDVerbose.m_oalLogFile, None, self.getTestName())
            
            
    def getPlugin( self ):
        """
        Returns the plugin instance
        """
        return self.getPluginChecker().getPlugin()
    
    
    def getErrorMessages( self ):
        """
        Returns the error messages for the plugin launcher
        """
        return self.getPluginChecker().getErrorMessages()
    
    
    def getWarningMessages( self ):
        """
        Returns the warning messages for the plugin launcher
        """
        return self.getPluginChecker().getWarningMessages()
            
            
    def setConfigurationFile( self, _edStringConfigurationFile ):
        """
        Sets the configuration file
        """
        self.m_edStringConfigurationFile = _edStringConfigurationFile
        
    
    def getConfigurationFile( self ):
        """
        Returns the configuration file
        """
        return self.m_edStringConfigurationFile
    
    
    def getRefConfigFile( self ):
        """
        Returns the reference configuration file (from edna/conf directory)
        """
        return self.m_edStringRefConfigFile
        
        
    def setDataInputFile( self, _edStringDataInputFile):  
        """
        Sets the data input file
        """
        self.m_edStringDataInputFile = _edStringDataInputFile
        
        
    def getDataInputFile( self ):
        """
        Returns the data input file
        """
        return self.m_edStringDataInputFile
    
    
    def setReferenceDataOutputFile( self, _edStringReferenceDataOutputFile ):
        """
        Sets the reference data output file (to check against obtained results)
        """
        self.m_edStringReferenceDataOutputFile = _edStringReferenceDataOutputFile
    
    
    def getReferenceDataOutputFile( self ):
        """
        Returns the reference data output file (to check against obtained results)
        """
        return self.m_edStringReferenceDataOutputFile
    
    
    def getEdnaSite( self ):
        """
        Returns the EDNA_SITE environment variable
        """
        return self.m_edStringEdnaSite
    
    
    def getPluginChecker( self ):
        """
        Returns the plugin checker
        """
        return self.m_edApplicationCheckPlugin
    
    
    def run( self ):
        """
        Executes the plugin launcher and checks that the data output is not None
        """
        self.m_edApplicationCheckPlugin.execute()
        dataOutputIsNone = True
        dataOutput = self.getPlugin().getDataOutput()
        if( dataOutput is not None ):
            dataOutputIsNone = False
            
        EDAssert.equal( False, dataOutputIsNone )
           
    

    def loadTestImage( self, _edListImageFileName ):
        """
        This method checks the presence of all the images in the list of image file names
        in the $EDNA_HOME/tests/data/images directory. If one image is not present
        this method tries to download it from http://www.edna-site.org/data/tests/images
        """
        for edStringImageName in _edListImageFileName:
            edStringImagePath = EDDiskExplorer.mergePath( self.getTestsDataImagesHome(), edStringImageName )
            if( not EDDiskExplorer.existFile( edStringImagePath ) ):
                iMaxDownloadTime = 10      # Max download time per image before timeout exception
                EDVerbose.unitTest( "Trying to download image %s, timout set to %d s" % ( edStringImagePath, iMaxDownloadTime) )
                edStringCommand = EDString( "cd %s \n wget --tries 1 --timeout %d http://www.edna-site.org/data/tests/images/%s" % ( self.getTestsDataImagesHome(), iMaxDownloadTime, edStringImageName ) )
                PyOs.system( edStringCommand )
                if ( not EDDiskExplorer.existFile( edStringImagePath ) ):
                    raise RuntimeError, "Could not automatically download test images %r! \n \
                                         If you are behind a firewall, please set the environment variable http_proxy. \n \
                                         Otherwise please try to download the images manually from \n \
                                         http://www.edna-site.org/data/tests/images" % _edListImageFileName
                
