#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Marie-Francoise Incardona
# Revised by: Romeu Andre' Pieritz
#
#############################################################################



from ALObject                  import ALObject
from ALString                  import ALString
from ALVerbose                 import ALVerbose
from ALThreadingCondition      import ALThreadingCondition


class ALAssert( ALObject ):
    """
    This class is an object encapsulation of the python assert utilities
    """

    m_bIsAssertionMode = False
    m_oalThreadingConditionAssert = ALThreadingCondition()

    def __init__( self ):
        ALObject.__init__( self )


    def setAssertionModeOn():
        """
        The invocation of this method enables the Assert checks
        i.e the invocation of the methods of this class will be executed otherwise
        it returns without the assert checks
        """

        ALAssert.m_bIsAssertionMode = True
        ALVerbose.setVerboseOn()
    setAssertionModeOn = staticmethod( setAssertionModeOn )


    def setAssertionModeOff():
        """
        The invocation of this method disables the Assert checks
        i.e the invocation of the methods won't execute the assert checks.
        """
        ALAssert.m_bIsAssertionMode = False
        ALVerbose.setVerboseDebugOff()
    setAssertionModeOff = staticmethod( setAssertionModeOff )


    def isAssertionMode():
        """
        Checks wether the assertion mode is set or not
        If it is set the assert checks will be executed.
        """
        assertionMode = ALAssert.m_bIsAssertionMode
        return assertionMode
    isAssertionMode = staticmethod( isAssertionMode )


    def equal( _oExpected, _oObject ):
        """Fail if the two objects are unequal as determined by the '==' operator.
        """
        if ( ALAssert.isAssertionMode() == False ):
            return

        ALAssert.m_oalThreadingConditionAssert.synchronizeOn()

        try:

            if  ( _oExpected != _oObject ):

                EQUAL_ERROR_ASSERT_MESSAGE = \
                    ALAssert.getFailureMessage( ALString( _oExpected ), ALString( _oObject ) )
                ALVerbose.ASSERT( EQUAL_ERROR_ASSERT_MESSAGE )
                raise AssertionError, EQUAL_ERROR_ASSERT_MESSAGE
            else:
                ALVerbose.ASSERT( ALAssert.getSuccessMessage( ALString( _oExpected ), ALString( _oObject)))
        finally:
            ALAssert.m_oalThreadingConditionAssert.synchronizeOff()
    equal = staticmethod( equal )


    def raiseException( _oExpectedException, _callableObj, *_args, **_kwargs ):
        """Fail unless an exception of class excClass is thrown
           by callableObj when invoked with arguments _args and keyword
           arguments _kwargs. This method raises an AssertionError if
           no exception or an unexpected exception is catched.
        """

        if ( ALAssert.isAssertionMode() == False ):
            return

        ALAssert.m_oalThreadingConditionAssert.synchronizeOn()

        if hasattr( _oExpectedException, '__name__' ):
            excName = _oExpectedException.__name__
        else:
            excName = ALString( _oExpectedException )

        NO_EXCEPTION_ASSERT_ERROR_MESSAGE = None

        # trick for older python version...backcompatibility to python 2.4....
        try:
            try:
                _callableObj(*_args, **_kwargs)
                # Error Case: No-Exception
                NO_EXCEPTION_ASSERT_ERROR_MESSAGE = \
                    ALAssert.getFailureMessage( _oExpectedException.__name__ , "No Exception" )

                ALVerbose.ASSERT( NO_EXCEPTION_ASSERT_ERROR_MESSAGE )


                raise AssertionError, NO_EXCEPTION_ASSERT_ERROR_MESSAGE
            except _oExpectedException:
                # Expected Case
                ALVerbose.ASSERT( ALAssert.getSuccessMessage( \
                    _oExpectedException.__name__, _oExpectedException.__name__ ) )

                return
            except Exception, err:
                # Error Case: An unexpected error has occured (Assertion Error could
                # be catched from No-Exception error: This case must not be treated,
                # since it has already been)
                if( NO_EXCEPTION_ASSERT_ERROR_MESSAGE == None ):
                    UNEXPECTED_EXCEPTION_ASSERT_ERROR_MESSAGE = \
                    ALAssert.getFailureMessage( _oExpectedException.__name__, err.__class__.__name__ )
                    ALVerbose.ASSERT( UNEXPECTED_EXCEPTION_ASSERT_ERROR_MESSAGE )

                    raise AssertionError, UNEXPECTED_EXCEPTION_ASSERT_ERROR_MESSAGE

                # Raises either No Exception or Unexpected Assertion Error
                raise
        finally:
            ALAssert.m_oalThreadingConditionAssert.synchronizeOff()
    raiseException = staticmethod( raiseException )


    def getFailureMessage( _expected, _obtained ):
        return ALString( "*** FAILURE: Expected = " + _expected + " / Obtained = " + _obtained )
    getFailureMessage = staticmethod( getFailureMessage )


    def getSuccessMessage( _expected, _obtained ):
        return ALString( "OK: Expected = " + _expected + " / Obtained = " + _obtained )
    getSuccessMessage = staticmethod( getSuccessMessage )




