#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlCharacterisationv10.py 705 2009-05-13 09:52:07Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList
from EDImportLib import EDDict

from EDPluginControl import EDPluginControl
from EDMessage       import EDMessage
from EDUtilsFile     import EDUtilsFile

from XSDataMXv1 import XSDataCrystal
from XSDataMXv1 import XSDataSpaceGroup
from XSDataMXv1 import XSDataCollection
from XSDataMXv1 import XSDataIndexingInput
from XSDataMXv1 import XSDataCharacterisation
from XSDataMXv1 import XSDataIntegrationInput
from XSDataMXv1 import XSDataStrategyInput


class EDPluginControlCharacterisationv10( EDPluginControl ):
    """
    The Plugin that controls the whole characterisation.
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControl.__init__( self )
        self.setXSDataInputClass( XSDataCollection )
        self.m_edStringPluginIndexingName = "EDPluginControlIndexingv10"
        self.m_edPluginIndexing = None  
        self.m_edStringPluginIntegrationName = "EDPluginControlIntegrationv10"
        self.m_edPluginIntegration = None  
        self.m_edStringPluginStrategyName = "EDPluginControlStrategyv10"
        self.m_edPluginStrategy = None  
        self.m_xsDataCharacterisation = None


    def checkParameters( self ):
        """
        Checks the mandatory parameters
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                      "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getDiffractionPlan(), "diffractionPlan" )


    def preProcess( self, _edObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControl.preProcess( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.preProcess..." )
        
        self.m_edPluginIndexing    = self.loadPlugin( self.m_edStringPluginIndexingName   , "Indexing" )
        self.m_edPluginIntegration = self.loadPlugin( self.m_edStringPluginIntegrationName, "Integration" )
        self.m_edPluginStrategy    = self.loadPlugin( self.m_edStringPluginStrategyName   , "Strategy"    )
        
        if ( self.m_edPluginIndexing is not None ):
            EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.preProcess: " + self.m_edStringPluginIndexingName + " Found... setting Data Input")
            # create Data Input for indexing
            xsDataCollection   = self.getDataInput()
            xsDataSample       = xsDataCollection.getSample()
            xsDataSubWedgeList = xsDataCollection.getSubWedge()
            xsDataExperimentalCondition = xsDataSubWedgeList[0].getExperimentalCondition()
            
            xsDataIndexingInput = XSDataIndexingInput()
            xsDataIndexingInput.setDataCollection( xsDataCollection )
            xsDataIndexingInput.setExperimentalCondition( xsDataExperimentalCondition )
            
            xsDataDiffractionPlan = xsDataCollection.getDiffractionPlan()
            xsDataStringForcedSpaceGroup = xsDataDiffractionPlan.getForcedSpaceGroup()
            if ( xsDataStringForcedSpaceGroup is not None ):
                xsDataCrystal = XSDataCrystal()
                xsDataSpaceGroup = XSDataSpaceGroup()
                xsDataSpaceGroup.setName( xsDataStringForcedSpaceGroup )
                xsDataCrystal.setSpaceGroup( xsDataSpaceGroup )
                xsDataIndexingInput.setCrystal( xsDataCrystal )
            
            self.m_edPluginIndexing.setDataInput( xsDataIndexingInput )
            
            # Populate characterisation object
            self.m_xsDataCharacterisation = XSDataCharacterisation()
            self.m_xsDataCharacterisation.setDataCollection( XSDataCollection.parseString( xsDataCollection.marshal() ) )

          
          
    def process( self, _edObject = None ):
        """
        """
        EDPluginControl.process( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.process" )
        self.connectProcess( self.m_edPluginIndexing.executeSynchronous )
        self.m_edPluginIndexing.connectSUCCESS( self.doIndexingToIntegrationTransition )
        self.m_edPluginIndexing.connectFAILURE( self.doFailureActionIndexing )
        self.m_edPluginIntegration.connectSUCCESS( self.doIntegrationToStrategyTransition )
        self.m_edPluginIntegration.connectFAILURE( self.doFailureActionIntegration )
        self.m_edPluginStrategy.connectFAILURE( self.doFailureActionStrategy )
        self.m_edPluginStrategy.connectSUCCESS( self.doSuccessActionStrategy )
        
        
    def postProcess( self, _edObject = None ):
        """
        """
        EDPluginControl.postProcess( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.postProcess" )
        if ( self.m_xsDataCharacterisation is not None ):
            self.setDataOutput( self.m_xsDataCharacterisation )
            
    
    def doIndexingToIntegrationTransition( self,  _edPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.doIndexingToIntegrationTransition" )
        if ( _edPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doFailureActionIntegration( _edPlugin )
        else:
            self.retrieveSuccessMessages( _edPlugin, "EDPluginControlCharacterisationv10.doIntegrationToStrategyTransition" )
            xsDataIndexingResult = self.m_edPluginIndexing.getDataOutput()
            self.m_xsDataCharacterisation.setIndexingResult( xsDataIndexingResult )
            xsDataIntegrationInput = XSDataIntegrationInput()
            xsDataIntegrationInput.setDataCollection( self.m_xsDataCharacterisation.getDataCollection() )
            xsDataIntegrationInput.setExperimentalConditionRefined( xsDataIndexingResult.getSelectedSolution().getExperimentalConditionRefined() )
            xsDataIntegrationInput.setSelectedIndexingSolution( xsDataIndexingResult.getSelectedSolution() )
            self.m_edPluginIntegration.setDataInput( xsDataIntegrationInput )
            self.m_edPluginIntegration.executeSynchronous()
        
        
    def doIntegrationToStrategyTransition( self,  _edPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.doIntegrationToStrategyTransition" )
        if ( _edPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doFailureActionIntegration( _edPlugin )
        else:
            self.retrieveSuccessMessages( _edPlugin, "EDPluginControlCharacterisationv10.doIntegrationToStrategyTransition" )
        
            xsDataIntegrationOutput = self.m_edPluginIntegration.getDataOutput()
            self.m_xsDataCharacterisation.setIntegrationResult( xsDataIntegrationOutput )
            #EDVerbose.DEBUG( self.m_xsDataExperimentCharacterisation.marshal() )
            xsDataStrategyInput = XSDataStrategyInput()
    
            xsDataSolutionSelected = self.m_xsDataCharacterisation.getIndexingResult().getSelectedSolution()
            
            xsDataStrategyInput.setCrystalRefined( xsDataSolutionSelected.getCrystal() )
            xsDataStrategyInput.setSample( self.m_xsDataCharacterisation.getDataCollection().getSample() )
                    
            xsDataIntegrationSubWedgeResultList = xsDataIntegrationOutput.getIntegrationSubWedgeResult()
            
            xsDataStrategyInput.setBestFileContentDat(    xsDataIntegrationSubWedgeResultList[0].getBestfileDat() )
            xsDataStrategyInput.setBestFileContentPar(    xsDataIntegrationSubWedgeResultList[0].getBestfilePar() )
            xsDataStrategyInput.setExperimentalCondition( xsDataIntegrationSubWedgeResultList[0].getExperimentalConditionRefined() )
            
            for xsDataIntegrationSubWedgeResult in xsDataIntegrationSubWedgeResultList:
                xsDataStrategyInput.addBestFileContentHKL( xsDataIntegrationSubWedgeResult.getBestfileHKL() )
                
            xsDataStrategyInput.setDiffractionPlan( self.m_xsDataCharacterisation.getDataCollection().getDiffractionPlan() )
            
            #print xsDataStrategyInput.marshal()
            self.m_edPluginStrategy.setDataInput( xsDataStrategyInput )
            self.m_edPluginStrategy.executeSynchronous()
        


    def doFailureActionIndexing( self,  _edPlugin = None ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.doFailureActionIndexing" )
        self.retrieveFailureMessages( _edPlugin, "EDPluginControlCharacterisationv10.doFailureActionIndexing" )        


    def doFailureActionIntegration( self,  _edPlugin = None ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.doFailureActionIntegration" )
        self.retrieveFailureMessages( _edPlugin, "EDPluginControlCharacterisationv10.doFailureActionIntegration" )


    def doFailureActionStrategy( self,  _edPlugin = None ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.doFailureActionStrategy" )
        self.retrieveFailureMessages( _edPlugin, "EDPluginControlCharacterisationv10.doFailureActionStrategy" )
        
        
    def doSuccessActionStrategy( self, _edPlugin ):
        """
        retrieve the potential warning messages
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.doSuccessActionStrategy" )
        if ( _edPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doFailureActionStrategy( _edPlugin )
        else:
            self.retrieveSuccessMessages( _edPlugin, "EDPluginControlCharacterisationv10.doSuccessActionStrategy" )
            xsDataStrategyResult = self.m_edPluginStrategy.getDataOutput()
            self.m_xsDataCharacterisation.setStrategyResult( xsDataStrategyResult )
      
      
      
    def generateExecutiveSummary( self, _edPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControlCharacterisationv10.generateExecutiveSummary" )
        self.addExecutiveSummaryLine( "Summary of characterisation:" )
        self.addErrorWarningMessagesToExecutiveSummary( "Characterisation failure! Error messages: " )
        self.addExecutiveSummarySeparator()
        xsDataCollection = self.getDataInput()
        xsDataDiffractionPlan = xsDataCollection.getDiffractionPlan()
        self.addExecutiveSummaryLine( "Diffraction plan:" )
        if ( xsDataDiffractionPlan.getComplexity() is not None ):
            self.addExecutiveSummaryLine( "BEST complexity                       : %s" % xsDataDiffractionPlan.getComplexity().getValue() )
        if ( xsDataDiffractionPlan.getAimedCompleteness() is not None ):
            self.addExecutiveSummaryLine( "Aimed completeness                    : %6.1f [%%]" % ( 100.0 * xsDataDiffractionPlan.getAimedCompleteness().getValue() ) )
        if ( xsDataDiffractionPlan.getRequiredCompleteness() is not None ):
            self.addExecutiveSummaryLine( "Required completeness                 : %6.1f [%%]" % ( 100.0 * xsDataDiffractionPlan.getRequiredCompleteness().getValue() ) )                  
        if ( xsDataDiffractionPlan.getAimedIOverSigmaAtHighestResolution() is not None ):
            self.addExecutiveSummaryLine( "Aimed I/sigma at highest resolution   : %6.1f" % xsDataDiffractionPlan.getAimedIOverSigmaAtHighestResolution().getValue() )                  
        if ( xsDataDiffractionPlan.getAimedResolution() is not None ):
            self.addExecutiveSummaryLine( "Aimed resolution                      : %6.1f [A]" % xsDataDiffractionPlan.getAimedResolution().getValue() )                  
        if ( xsDataDiffractionPlan.getRequiredResolution() is not None ):
            self.addExecutiveSummaryLine( "Required resolution                   : %6.1f [A]" % xsDataDiffractionPlan.getRequiredResolution().getValue() )                  
        if ( xsDataDiffractionPlan.getAimedMultiplicity() is not None ):
            self.addExecutiveSummaryLine( "Aimed multiplicity                    : %6.1f" % xsDataDiffractionPlan.getAimedMultiplicity().getValue() )                  
        if ( xsDataDiffractionPlan.getRequiredMultiplicity() is not None ):
            self.addExecutiveSummaryLine( "Required multiplicity                 : %6.1f" % xsDataDiffractionPlan.getRequiredMultiplicity().getValue() )                  
        if ( xsDataDiffractionPlan.getForcedSpaceGroup() is not None ):
            self.addExecutiveSummaryLine( "Forced space group                    : %6s" % xsDataDiffractionPlan.getForcedSpaceGroup().getValue() )                  
        if ( xsDataDiffractionPlan.getMaxExposureTimePerDataCollection() is not None ):
            self.addExecutiveSummaryLine( "Max exposure time per data collection : %6.1f [s]" % xsDataDiffractionPlan.getMaxExposureTimePerDataCollection().getValue() )                  
        self.addExecutiveSummarySeparator()
        if ( self.m_edPluginIndexing is not None ):
            self.appendExecutiveSummary( self.m_edPluginIndexing, "Indexing : " )
        if ( self.m_edPluginIntegration is not None ):
            self.appendExecutiveSummary( self.m_edPluginIntegration, "Integration : " )
        if ( self.m_edPluginStrategy is not None ):
            self.appendExecutiveSummary( self.m_edPluginStrategy, "Strategy : " )
            
      