#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlGeneratePredictionv10.py 748 2009-05-29 11:38:00Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList
from EDImportLib import EDDict
from EDImportLib import EDActionCluster

from EDPluginControl import EDPluginControl
from EDMessage       import EDMessage
from EDUtilsFile     import EDUtilsFile
from EDConfiguration import EDConfiguration

from XSDataCommon import XSDataFloat
from XSDataCommon import XSDataImage

from XSDataMXv1 import XSDataCollection
from XSDataMXv1 import XSDataSubWedge
from XSDataMXv1 import XSDataExperimentalCondition
from XSDataMXv1 import XSDataIndexingSolutionSelected
from XSDataMXv1 import XSDataExperimentalCondition
from XSDataMXv1 import XSDataIntegrationResult
from XSDataMXv1 import XSDataGeneratePredictionInput
from XSDataMXv1 import XSDataGeneratePredictionResult


class EDPluginControlGeneratePredictionv10( EDPluginControl ):
    """
    The Plugin that control the plugin that generates the prediction.
    If several images are available the execution of the plugin which generates
    the prediction is performed in parallel.
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControl.__init__( self )
        self.m_edStringPluginGeneratePredictionName = "EDPluginMOSFLMGeneratePredictionv10"
        #self.m_edStringPluginGeneratePredictionName = "EDPluginMOSFLMv01GeneratePrediction"
        self.m_edPluginGeneratePredictionList = []
        self.m_dNoSubwedges = None
        self.m_xsDataGeneratePredictionResult = None
        self.setXSDataInputClass( XSDataGeneratePredictionInput )


    def checkParameters( self ):
        """
        Checks the mandatory parameters
        """
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                               "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getDataCollection(),           "dataCollection" )
        self.checkMandatoryParameters( self.getDataInput().getSelectedIndexingSolution(), "selectedIndexingSolution" )


    def preProcess( self, _edObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControl.preProcess( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv10.preProcess..." )
        
        self.m_edActionCluster = EDActionCluster()
        # TODO: Value to be read from the configuration... see bug #95
        self.m_edActionCluster.setClusterSize(4)

        xsDataGeneratePredictionInput = self.getDataInput()       

        xsDataSelectedIndexingSolution     = xsDataGeneratePredictionInput.getSelectedIndexingSolution()
        xsDataExperimentalConditionRefined = xsDataSelectedIndexingSolution.getExperimentalConditionRefined()
        xsDataCollection                   = xsDataGeneratePredictionInput.getDataCollection()
        xsDataSubWedgeList                 = xsDataCollection.getSubWedge()
        
        # List containing instances of all the generate prediction plugins
        self.m_edPluginGeneratePredictionList = EDList()
        
        # Loop through all subwedges
        iIndex = 0
        for xsDataSubWedge in xsDataSubWedgeList:

            xsDataImageList = xsDataSubWedge.getImage()
            # First find the lowest image number
            iLowestImageNumber = None
            for xsDataImage in xsDataImageList:
                iImageNumber = xsDataImage.getNumber().getValue()
                if ( iLowestImageNumber is None ):
                    iLowestImageNumber = iImageNumber
                elif ( iImageNumber < iLowestImageNumber ):
                    iLowestImageNumber = iImageNumber                
            # Then loop through all images in a sub wedge
            for xsDataImage in xsDataImageList:
                iIndex += 1
                edPluginGeneratePrediction = self.loadPlugin( self.m_edStringPluginGeneratePredictionName, 
                                                              "%s-%02d" % ( self.m_edStringPluginGeneratePredictionName, iIndex) )
                xsDataGeneratePredictionInput = XSDataGeneratePredictionInput()
                xsDataGeneratePredictionInput.setSelectedIndexingSolution( XSDataIndexingSolutionSelected.parseString( xsDataSelectedIndexingSolution.marshal() )  )
                xsDataCollectionNew = XSDataCollection()
                xsDataSubWedgeNew = XSDataSubWedge()
                xsDataSubWedgeNew.addImage( XSDataImage.parseString( xsDataImage.marshal() ) )
                xsDataSubWedgeNew.setExperimentalCondition( XSDataExperimentalCondition.parseString( xsDataSubWedge.getExperimentalCondition().marshal() ) )
                # We must modify the rotationOscillationStart for the new subwedge
                xsDataGoniostatNew = xsDataSubWedgeNew.getExperimentalCondition().getGoniostat()
                fGoniostatRotationAxisStart = xsDataGoniostatNew.getRotationAxisStart().getValue()
                fGonioStatOscillationRange  = xsDataGoniostatNew.getOscillationWidth().getValue()
                iImageNumber = xsDataImage.getNumber().getValue()
                fGoniostatRotationAxisStartNew = fGoniostatRotationAxisStart + ( iImageNumber - iLowestImageNumber )*fGonioStatOscillationRange
                xsDataGoniostatNew.setRotationAxisStart( XSDataFloat( fGoniostatRotationAxisStartNew ) )
                # 
                xsDataCollectionNew.addSubWedge( xsDataSubWedgeNew )
                xsDataGeneratePredictionInput.setDataCollection( xsDataCollectionNew )
                from EDHandlerXSDataMOSFLMv10 import EDHandlerXSDataMOSFLMv10
                try:
                    xsDataMOSFLMInputGeneratePrediction= EDHandlerXSDataMOSFLMv10.generateXSDataMOSFLMInputGeneratePrediction( xsDataGeneratePredictionInput )
                except Exception, error:
                    errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginControlGeneratePredictionv10.preProcess: Unexpected error in MOSFLM handler", error )
                    EDVerbose.error( errorMessage )
                    self.addErrorMessage( errorMessage )
                    raise Exception, errorMessage
                edPluginGeneratePrediction.setDataInput( xsDataMOSFLMInputGeneratePrediction )
                self.m_edPluginGeneratePredictionList.add( edPluginGeneratePrediction )
                    
          
          
    def process( self, _edObject = None ):
        """
        """
        EDPluginControl.process( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv10.process" )
        # Prepare the action cluster
        for edPluginGeneratePrediction in self.m_edPluginGeneratePredictionList:
            edPluginGeneratePrediction.connectSUCCESS( self.doSuccessGeneratePrediction )
            edPluginGeneratePrediction.connectFAILURE( self.doFailureGeneratePrediction )
            self.m_edActionCluster.addAction( edPluginGeneratePrediction )
        # Launch the cluster
        self.m_edActionCluster.executeAction()
        self.m_edActionCluster.synchronize()
        
        
    def doSuccessGeneratePrediction( self,  _edPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv10.doSuccessGeneratePrediction" )
        if ( _edPlugin.getErrorMessages().getNumberObjects() != 0 ):
            EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv10.doSuccessGeneratePrediction: Execution plugin returned error messages, failure!" )
            self.doFailureGeneratePrediction( _edPlugin )
        else:
            self.retrieveSuccessMessages( _edPlugin, "EDPluginControlGeneratePredictionv10.doSuccessGeneratePrediction" )
            if ( self.m_xsDataGeneratePredictionResult is None ):
                self.m_xsDataGeneratePredictionResult = XSDataGeneratePredictionResult()
            xsDataMOSFLMOutputGeneratePrediction = _edPlugin.getDataOutput()
            xsDataImage = xsDataMOSFLMOutputGeneratePrediction.getPredictionImage()
            self.m_xsDataGeneratePredictionResult.addPredictionImage( XSDataImage.parseString( xsDataImage.marshal() ))


    def doFailureGeneratePrediction( self,  _edPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv10.doFailureGeneratePrediction" )
        self.retrieveFailureMessages( _edPlugin, "EDPluginControlGeneratePredictionv10.doFailureGeneratePrediction" )


    def postProcess( self, _edPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv10.postProcess" )
        self.setDataOutput( self.m_xsDataGeneratePredictionResult )


    def generateExecutiveSummary( self, _edPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginControlGeneratePredictionv10.generateExecutiveSummary" )
        self.addExecutiveSummaryLine( "Summary of generation of image(s) with prediction:" )
        self.addErrorWarningMessagesToExecutiveSummary( "Generation of prediction failure! Error messages: " )
        xsDataGeneratePredictionResult = self.getDataOutput()
        if ( xsDataGeneratePredictionResult is not None ):
            for predictionImage in xsDataGeneratePredictionResult.getPredictionImage():
                self.addExecutiveSummaryLine( "Path to prediction image number %d : %s" % \
                                             ( predictionImage.getNumber().getValue(), predictionImage.getPath().getValue() ) )
            