#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlIndexingv10.py 701 2009-05-13 09:14:46Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList

from EDPluginControl import EDPluginControl
from EDMessage       import EDMessage
from EDUtilsFile     import EDUtilsFile
from EDConfiguration import EDConfiguration

from XSDataCommon import XSPluginItem
from XSDataCommon import XSDataImage

from XSDataMXv1 import XSDataCollection
from XSDataMXv1 import XSDataCrystal
from XSDataMXv1 import XSDataSubWedge
from XSDataMXv1 import XSDataExperimentalCondition
from XSDataMXv1 import XSDataIndexingInput
from XSDataMXv1 import XSDataGeneratePredictionInput
from XSDataMXv1 import XSDataIndexingSolutionSelected


class EDPluginControlIndexingv10( EDPluginControl ):
    """
    The plugin which controls the indexing execution plugins.
    This plugin accepts several sub-wedges with one or several images each.
    The indexing is performed on all images together.
    One image with prediction is generated for each input image.
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControl.__init__( self )      
        self.setXSDataInputClass( XSDataIndexingInput ) 
        self.m_edStringPluginIndexingName = "EDPluginControlIndexingWrapperMOSFLMv10"
        #self.m_edStringPluginIndexingName = "EDPluginControlIndexingWrapperLabelitv10"
        #self.m_edStringPluginIndexingName = "EDPluginControlIndexingParallelv10"
        self.m_edPluginIndexing = None
        self.m_edStringPluginGeneratePredictionName = "EDPluginControlGeneratePredictionv10"
        self.m_edPluginGeneratePrediction = None
        self.m_xsDataIndexingOutput = None
        self.m_xsDataExperimentalCondition = None
        self.m_xsDataIndexingResult = None      
        self.m_xsDataCollection = None
        self.m_bGeneratePredictionImage = False
        self.m_edStringPluginIndexingExecutiveSummaryName = "Control"


    def checkParameters( self ):
        """
        Checks the mandatory parameters
        """
        self.verboseDebug( "*** EDPluginControlIndexingv10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                     "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getDataCollection(), "dataCollection" )


    def preProcess( self, _edObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControl.preProcess( self, _edObject )
        self.verboseDebug( "*** EDPluginControlIndexingv10.preProcess..." )
        if ( self.m_edStringPluginIndexingName is not None ):
            # Load the indexing plugin and generate input data
            self.verboseDebug( "*** EDPluginControlIndexingv10.preProcess : loading plugin %s" % self.m_edStringPluginIndexingName )
            self.m_edPluginIndexing = self.loadPlugin( self.m_edStringPluginIndexingName )
            self.loadPluginIndexingInputData()
            xsDataIndexingInput = self.getDataInput()
            # Save the data collection and the experimental condition for constructing the indexing results
            self.m_xsDataExperimentalCondition = xsDataIndexingInput.getExperimentalCondition()
            self.m_xsDataCollection = xsDataIndexingInput.getDataCollection()
        # Load the generate prediction plugin
        self.m_edPluginGeneratePrediction = self.loadPlugin( self.m_edStringPluginGeneratePredictionName )
        
        
    def loadPluginIndexingInputData( self ):
        self.verboseDebug( "*** EDPluginControlIndexingv10.loadPluginIndexingInputData..." )
        self.m_edPluginIndexing.setDataInput( self.getDataInput()  )
        
          
    def process( self, _edObject = None ):
        """
        """
        EDPluginControl.process( self, _edObject )
        self.verboseDebug( "*** EDPluginControlIndexingv10.process" )
        if ( self.m_edPluginIndexing is not None ):
            self.m_edPluginIndexing.connectSUCCESS( self.doSuccessActionIndexing )
            self.m_edPluginIndexing.connectFAILURE( self.doFailureActionIndexing )
            self.connectProcess( self.m_edPluginIndexing.executeSynchronous )
        self.m_edPluginGeneratePrediction.connectSUCCESS( self.doSuccessActionGeneratePrediction )
        self.m_edPluginGeneratePrediction.connectFAILURE( self.doFailureActionGeneratePrediction )
        
        
    def postProcess( self, _edObject = None ):
        EDPluginControl.postProcess( self, _edObject )
        self.verboseDebug( "*** EDPluginControlIndexingv10.postProcess" )
        self.setDataOutput( self.m_xsDataIndexingResult )  


    def doSuccessActionIndexing( self, _edPlugin = None ):
        self.verboseDebug( "*** EDPluginControlIndexingv10.doSuccessActionIndexing" )
        self.retrieveSuccessMessages( _edPlugin, "EDPluginControlIndexingv10.doSuccessActionIndexing" )
        # Retrieve the output from the plugin
        self.m_xsDataIndexingResult = self.getDataIndexingResult( _edPlugin )    
        # Add the list of images to the results
        xsDataListImage = self.generateImageList( self.m_xsDataCollection ) 
        self.m_xsDataIndexingResult.setImage( xsDataListImage )
        if ( self.m_bGeneratePredictionImage ):
            # Generate prediction images
            xsDataGeneratePredictionInput = XSDataGeneratePredictionInput()
            xsDataGeneratePredictionInput.setDataCollection( XSDataCollection.parseString( self.m_xsDataCollection.marshal() ) )
            xsDataGeneratePredictionInput.setSelectedIndexingSolution( XSDataIndexingSolutionSelected.parseString( self.m_xsDataIndexingResult.getSelectedSolution().marshal() ) )
            self.m_edPluginGeneratePrediction.setDataInput( xsDataGeneratePredictionInput )
            self.m_edPluginGeneratePrediction.executeSynchronous()


    def getDataIndexingResult( self, _edPlugin ):
        """
        This method retrieves the indexing results from a MOSFLM indexing plugin.
        """
        self.verboseDebug( "*** EDPluginControlIndexingv10.getDataIndexingResult" )
        return _edPlugin.getDataOutput()


    def doFailureActionIndexing( self,  _edPlugin = None ):
        self.verboseDebug( "*** EDPluginControlIndexingv10.doFailureActionIndexing" )
        EDVerbose.screen( "Execution of " + self.m_edStringPluginIndexingName + "  failed." )
        EDVerbose.screen( "Please inspect the log file for further information." )
        self.retrieveFailureMessages( _edPlugin, "EDPluginControlIndexingv10.doFailureActionIndexing" )

    

    def generatePredictionImageList( self, _edPluginGeneratePrediction, _xsDataCollection, _xsDataIndexingResult ):
        """
        Generate a list containing the prediction images
        """
        self.verboseDebug( "*** EDPluginControlIndexingv10.generatePredictionImageList" )
        xsDataGeneratePredictionInput = XSDataGeneratePredictionInput()
        xsDataGeneratePredictionInput.setDataCollection( XSDataCollection.parseString( _xsDataCollection.marshal() ) )
        xsDataGeneratePredictionInput.setSelectedIndexingSolution( XSDataIndexingSolutionSelected.parseString( _xsDataIndexingResult.getSelectedSolution().marshal() ) )
        _edPluginGeneratePrediction.setDataInput( xsDataGeneratePredictionInput )
        _edPluginGeneratePrediction.executeSynchronous()
        
        
        
    def doSuccessActionGeneratePrediction( self, _edPlugin ):
        """
        Executed if the generate prediction plugin terminated correctly.
        """
        self.verboseDebug( "*** EDPluginControlIndexingv10.doSuccessActionGeneratePrediction" )
        xsDataImageList = None
        if ( _edPlugin.getErrorMessages().getNumberObjects() != 0 ):
            # Translate all error messages to warning messages
            for errorMessage in _edPlugin.getErrorMessages():
                warningMessage = EDMessage.WARNING_CANNOT_USE_PLUGIN_03 % ( "EDPluginControlIndexingv10.generatePredictionImageList", \
                                                                            self.m_edStringPluginGeneratePredictionName, \
                                                                            errorMessage  )
                EDVerbose.warning( warningMessage )
                self.addWarningMessages( [ warningMessage ] )
        else:
            self.retrieveSuccessMessages(_edPlugin, "EDPluginControlIndexingv10.doSuccessActionGeneratePrediction" )               
            xsDataGeneratePredictionResult = _edPlugin.getDataOutput()
            self.m_xsDataIndexingResult.setPredictionResult( xsDataGeneratePredictionResult )
    
    
    def doFailureActionGeneratePrediction( self, _edPlugin ):
        """
        Executed if the generate prediction plugin did not terminate correctly.
        """
        self.verboseDebug( "*** EDPluginControlIndexingv10.doFailureActionGeneratePrediction" )
        warningMessage = EDMessage.WARNING_CANNOT_USE_PLUGIN_03 % ( "EDPluginControlIndexingv10.doFailureActionGeneratePrediction", \
                                                                    self.m_edStringPluginGeneratePredictionName, \
                                                                    errorMessage  )
        EDVerbose.warning( warningMessage )
        self.addWarningMessages( [ warningMessage ] )

    
    
    
    def generateImageList( self, _xsDataCollection ):
        """
        Make a list of all images in the subwedges
        """
        self.verboseDebug( "*** EDPluginControlIndexingv10.generateImageList" )
        edListImage = None
        if ( _xsDataCollection is not None ):
            edListImage = EDList()
            xsDataSubWedgeList = _xsDataCollection.getSubWedge()
            for xsDataSubWedge in xsDataSubWedgeList:
                xsDataImageList = xsDataSubWedge.getImage()
                for xsDataImage in xsDataImageList:
                    edListImage.add( XSDataImage.parseString( xsDataImage.marshal() ) )   
        return edListImage
    

    def generateExecutiveSummary( self, _edPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        self.verboseDebug( "*** EDPluginControlIndexingv10.generateExecutiveSummary" )
        self.addExecutiveSummaryLine( "Summary of indexing with %s :" % self.m_edStringPluginIndexingName )
        self.addErrorWarningMessagesToExecutiveSummary( "Indexing failure! Error messages: " )
        if ( self.m_edPluginIndexing is not None ):
            self.appendExecutiveSummary( self.m_edPluginIndexing, self.m_edStringPluginIndexingExecutiveSummaryName + " : ")
        if ( self.m_edPluginGeneratePrediction is not None ):
            self.appendExecutiveSummary( self.m_edPluginGeneratePrediction, "Prediction : " )
        