#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlSubWedgeAssemblev10.py 698 2009-05-11 10:43:08Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal author:      Olof Svensson (svensson@esrf.fr)
#
#    Contributing authors:   Marie-Francoise Incardona (incardon@esrf.fr)
#                            Gleb Bourenkov (Gleb.Bourenkov@embl-hamburg.de)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList

from EDMessage       import EDMessage
from EDPluginControl import EDPluginControl

from XSDataCommon import XSDataFile

from XSDataMXv1 import XSDataInputSubWedgeMerge
from XSDataMXv1 import XSDataInputSubWedgeAssemble
from XSDataMXv1 import XSDataResultSubWedgeAssemble
from XSDataMXv1 import XSDataInputReadImageHeader


class EDPluginControlSubWedgeAssemblev10( EDPluginControl ):
    """
    This plugin takes as input a list of paths to images. It executes the 
    EDPluginReadImageHeaderv10 plugin for each image path in order to read 
    the image header data and produce a XSDataSubWedge instance for each image.
    
    It then executes the EDPluginSubWedgeMergev10 plugin for, if possible, 
    merging these subwedges. 
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControl.__init__( self )    
        self.setXSDataInputClass( XSDataInputSubWedgeAssemble )
        self.m_edStringPluginReadImageHeaderName = "EDPluginControlReadImageHeaderv10"
        self.m_edPluginReadImageHeader           = None
        self.m_edStringPluginSubWedgeMergeName   = "EDPluginSubWedgeMergev10"
        self.m_edPluginSubWedgeMerge             = None
        self.m_xsDataResultSubWedgeMerge         = None


    def checkParameters( self ):
        """
        Checks the mandatory parameters
        """
        EDVerbose.DEBUG( "*** EDPluginControlSubWedgeAssemblev10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                     "Data Input is None" )
        self.checkMandatoryParameters( self.getDataInput().getFile(),           "file" )


    def preProcess( self, _edObject = None ):
        """
        Gets the Configuration Parameters, if found, overrides default parameters
        """
        EDPluginControl.preProcess( self, _edObject )
        EDVerbose.DEBUG( "*** EDPluginControlSubWedgeAssemblev10.preProcess")
        
        self.m_edPluginReadImageHeader = self.loadPlugin( self.m_edStringPluginReadImageHeaderName )        
        self.m_edPluginSubWedgeMerge   = self.loadPlugin( self.m_edStringPluginSubWedgeMergeName )
        
          
    def process( self, _oedObject = None ):
        """
        """
        EDPluginControl.process( self, _oedObject )
        EDVerbose.DEBUG( "*** EDPluginControlSubWedgeAssemblev10.process")
        if ( self.m_edPluginReadImageHeader is not None and self.m_edPluginSubWedgeMerge is not None ):
            self.connectProcess( self.createDataCollectionFromImageHeaders )
            self.m_edPluginSubWedgeMerge.connectSUCCESS( self.doSuccessActionSubWedgeMerge )
            self.m_edPluginSubWedgeMerge.connectFAILURE( self.doFailureActionSubWedgeMerge )
           
        
    def postProcess( self, _oedObject = None ):
        EDPluginControl.postProcess( self, _oedObject )
        EDVerbose.DEBUG( "*** EDPluginControlSubWedgeAssemblev10.postProcess")
        if ( self.m_xsDataResultSubWedgeMerge is not None ):
            xsDataResultSubWedgeAssemble = XSDataResultSubWedgeAssemble()
            xsDataResultSubWedgeAssemble.setSubWedge( self.m_xsDataResultSubWedgeMerge.getSubWedge() )
            self.setDataOutput( xsDataResultSubWedgeAssemble )
              


    def createDataCollectionFromImageHeaders( self, _edPlugin ):    
        """
        This method creates a list of XSDataSubwedges by reading the header of the images
        given as a list of paths as input.
        """          
        EDVerbose.DEBUG( "*** EDPluginControlSubWedgeAssemblev10.createDataCollectionFromImageHeaders")
        edListSubWedge = None
        xsDataInputSubWedgeAssemble = _edPlugin.getDataInput()
        edListXSDataFile = xsDataInputSubWedgeAssemble.getFile()
        if ( len( edListXSDataFile ) > 0 ):
            edListSubWedge = EDList()
            iIndex = 1
            for xsDataFile in edListXSDataFile:
                xsDataFileCopy = XSDataFile.parseString( xsDataFile.marshal() )
                xsDataInputReadImageHeader = XSDataInputReadImageHeader()
                xsDataInputReadImageHeader.setImage( xsDataFileCopy )
                self.m_edPluginReadImageHeader = self.loadPlugin( self.m_edStringPluginReadImageHeaderName, "ReadImageHeader_%d" % ( iIndex ) )
                self.m_edPluginReadImageHeader.setDataInput( xsDataInputReadImageHeader )
                self.m_edPluginReadImageHeader.executeSynchronous()
                xsDataResultReadImageHeader = self.m_edPluginReadImageHeader.getDataOutput()
                xsDataSubWedge = xsDataResultReadImageHeader.getSubWedge()
                if ( xsDataSubWedge is None ):
                    # Fix for bug #223: raise an error if an image could not be read
                    errorMessage = EDMessage.ERROR_EXECUTION_03 % ( "EDPluginControlSubWedgeAssemblev10.createDataCollectionFromImageHeaders", \
                                                                    self.m_edStringPluginReadImageHeaderName, \
                                                                    "Could not read header from image %s" % xsDataFileCopy.getPath().getValue() )
                    EDVerbose.error( errorMessage )          
                    self.addErrorMessage( errorMessage )              
                    raise RuntimeError, errorMessage                
                else:
                    edListSubWedge.add( xsDataSubWedge )
                iIndex += 1
            xsDataInputSubWedgeMerge = XSDataInputSubWedgeMerge()
            for xsDataSubWedge in edListSubWedge:
                xsDataInputSubWedgeMerge.addSubWedge( xsDataSubWedge )
            self.m_edPluginSubWedgeMerge.setDataInput( xsDataInputSubWedgeMerge )
            self.m_edPluginSubWedgeMerge.executeSynchronous()
        

    def doSuccessActionSubWedgeMerge( self, _edPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlSubWedgeAssemblev10.doSuccessActionSubWedgeMerge" )
        self.retrieveSuccessMessages( _edPlugin, "EDPluginControlSubWedgeAssemblev10.doSuccessActionSubWedgeMerge" )
        if ( _edPlugin.getErrorMessages().getNumberObjects() == 0 ):
            self.m_xsDataResultSubWedgeMerge = _edPlugin.getDataOutput()   


    def doFailureActionSubWedgeMerge( self,  _edPlugin = None ):
        EDVerbose.DEBUG( "*** EDPluginControlSubWedgeAssemblev10.doFailureActionSubWedgeMerge" )
        EDVerbose.screen( "Execution of " + self.m_edStringPluginSubWedgeMergeName + "  failed." )
        EDVerbose.screen( "Please inspect the log file for further information." )
        self.retrieveFailureMessages( _edPlugin, "EDPluginControlSubWedgeAssemblev10.doFailureActionSubWedgeMerge" )

        