#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlReadImageHeaderv10.py 705 2009-05-13 09:52:07Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDString
from EDImportLib import EDVerbose
from EDImportLib import EDDictionary

from EDUtilsFile     import EDUtilsFile
from EDMessage       import EDMessage
from EDPluginControl import EDPluginControl

from XSDataMXv1 import XSDataInputReadImageHeader

class EDPluginControlReadImageHeaderv10( EDPluginControl ):


    def __init__( self ):
        """
        """
        EDPluginControl.__init__( self )
        self.setXSDataInputClass( XSDataInputReadImageHeader )
        self.m_edPluginExecReadImageHeader       = None
        self.m_edStringReadImageHeaderPluginName = None
        self.m_xsDataResultReadImageHeader       = None
        # Map between image suffix and image type
        self.m_edStringSuffixADSC                = EDString( "img" )
        self.m_edStringSuffixMARCCD1             = EDString( "mccd" )
        self.m_edStringSuffixMARCCD2             = EDString( "marccd" )
        # Recognised types of detectors
        self.m_edStringADSC                      = EDString( "ADSC" )
        self.m_edStringMARCCD                    = EDString( "MARCCD" )
        #
        self.m_edStringPluginExecReadImageHeaderADSC   = EDString( "EDPluginExecReadImageHeaderADSCv10" )
        self.m_edStringPluginExecReadImageHeaderMARCCD = EDString( "EDPluginExecReadImageHeaderMARCCDv10" )
        # Table mapping image suffix with detector type
        self.m_edDictionarySuffixToImageType = EDDictionary( { \
              self.m_edStringSuffixADSC    : self.m_edStringADSC, \
              self.m_edStringSuffixMARCCD1 : self.m_edStringMARCCD, \
              self.m_edStringSuffixMARCCD2 : self.m_edStringMARCCD \
                                                    } )
		# Table mapping image type with exec read image header plugin
        self.m_edDictionaryImageTypeToPluginName = EDDictionary( { \
			  self.m_edStringADSC   : self.m_edStringPluginExecReadImageHeaderADSC, \
              self.m_edStringMARCCD : self.m_edStringPluginExecReadImageHeaderMARCCD
                									 } )

    def checkParameters( self ):
        """
        Checks the mandatory parameters.
        """
        EDVerbose.DEBUG( "*** EDPluginControlReadImageHeaderv10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(), "Data Input is None" )


    def preProcess( self, _edObject = None ):
        """
        """
        EDPluginControl.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginControlReadImageHeaderv10.preProcess")
        # First determine the type of image
        xsDataInputReadImageHeader        = self.getDataInput()
        xsDataFileImage                   = xsDataInputReadImageHeader.getImage()
        edStringFileImagePath             = EDString( xsDataFileImage.getPath().getValue() )
        edStringImageType                 = self.determineImageType( edStringFileImagePath )
        edStringReadImageHeaderPluginName = self.determineExecReadImageHeaderPluginName( edStringImageType )
        # Will be changed once code review #240 is ready
        self.m_edPluginExecReadImageHeader = self.loadPlugin( edStringReadImageHeaderPluginName )
        self.m_edPluginExecReadImageHeader.setDataInput( xsDataInputReadImageHeader )
            
        
    def process( self, _edObject = None ):
        """
        """
        EDPluginControl.process( self )
        EDVerbose.DEBUG( "*** EDPluginControlReadImageHeaderv10.process")
        if ( self.m_edPluginExecReadImageHeader is not None ):
            self.m_edPluginExecReadImageHeader.connectSUCCESS( self.doSuccessActionReadImageHeader )
            self.m_edPluginExecReadImageHeader.connectFAILURE( self.doFailureActionReadImageHeader )
            self.m_edPluginExecReadImageHeader.executeSynchronous()


    def postProcess( self, _edObject = None ):
        """
        """
        EDPluginControl.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginControlReadImageHeaderv10.postProcess")
        if ( self.m_xsDataResultReadImageHeader is not None ):
            self.setDataOutput( self.m_xsDataResultReadImageHeader )


    def doSuccessActionReadImageHeader( self, _edPlugin ):
        """
        Retrieve the potential warning messages
        """
        EDVerbose.DEBUG( "*** EDPluginControlReadImageHeaderv10.doSuccessActionReadImageHeader" )
        self.retrieveSuccessMessages( _edPlugin, "EDPluginControlReadImageHeaderv10.doSuccessActionReadImageHeader" )
        self.m_xsDataResultReadImageHeader = _edPlugin.getDataOutput()
        
        
    def doFailureActionReadImageHeader( self, _edPlugin ):
        """
        Retrieve the potential warning messages
        Retrieve the potential error messages
        """
        EDVerbose.DEBUG( "*** EDPluginControlReadImageHeaderv10.doFailureActionReadImageHeader" )
        self.retrieveFailureMessages( _edPlugin, "EDPluginControlReadImageHeaderv10.doFailureActionReadImageHeader" )
        
        
    def determineImageType( self, _edStringImagePath ):
        """
        This method determines the type of an image, i.e. ADSC, MAR CCD etc.
        """
        edStringImageType = None
        EDVerbose.DEBUG( "*** EDPluginControlReadImageHeaderv10.determineImageType" )
        # For the moment, only look on the image suffix
        edStringImageSuffix = EDString( EDUtilsFile.getFileExtension( _edStringImagePath ) )
        if edStringImageSuffix in self.m_edDictionarySuffixToImageType.keys():
            edStringImageType = self.m_edDictionarySuffixToImageType[ edStringImageSuffix ]
        else:
            edStringMessage = EDString( "Unknown image type for image %s " % _edStringImagePath )
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ( "EDPluginControlReadImageHeaderv10.preProcess", \
                                                            self.getClassName(),
                                                            edStringMessage )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage            
        return edStringImageType
    
    
    def determineExecReadImageHeaderPluginName( self, _edStringImageType ):
        """
        This method determines the name of the exec plugin read image header
        """
        edStringReadImageHeaderPluginName = None
        EDVerbose.DEBUG( "*** EDPluginControlReadImageHeaderv10.determinePluginName" )
        # For the moment, only look on the image suffix
        if _edStringImageType in self.m_edDictionaryImageTypeToPluginName.keys():
            edStringReadImageHeaderPluginName = self.m_edDictionaryImageTypeToPluginName[ _edStringImageType ]
        else:
            edStringMessage = EDString( "EDPluginControlReadImageHeaderv10.preProcess: Unknown image type: %s" % \
                                        _edStringImageType )
            errorMessage = EDMessage.ERROR_UNEXPECTED_01 % ( edStringMessage )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage
        
        return edStringReadImageHeaderPluginName
    