#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id:$"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from EDImportSystem import PyOs

from EDImportLib import EDVerbose
from EDImportLib import EDCompiler
from EDImportLib import EDString
from EDImportLib import EDList
from EDImportLib import EDDiskExplorer
from EDImportLib import EDTime

from EDMessage import EDMessage
from EDConfiguration import EDConfiguration
from EDApplicationDeprecated import EDApplicationDeprecated

from XSDataCommon import XSDataFile
from XSDataCommon import XSDataString
from XSDataCommon import XSDataTime
from XSDataCommon import XSDataFloat
from XSDataCommon import XSDataSize
from XSDataCommon import XSDataLength

from XSDataMXv1 import XSDataDiffractionPlan
from XSDataMXv1 import XSDataCollection

from XSDataMXv1 import XSDataInputSubWedgeAssemble


class EDApplicationMXv1Characterisation( EDApplicationDeprecated ):

    m_edStringAPPLICATION_NAME              = "EDApplicationMXv1Characterisation"
    m_edStringAPPLICATION_VERSION           = "1.0"
    m_edStringIMAGE_PARAM_LABEL             = "--image"
    m_edStringDATASET_PARAM_LABEL           = "--data"
    m_edStringFLUX_PARAM_LABEL              = "--flux"
    m_edStringBEAM_SIZE_PARAM_LABEL         = "--beamSize"
    m_edStringMAX_EXPOSURE_TIME_PARAM_LABEL = "--maxExposureTimePerDataCollection"
    m_edStringMIN_EXPOSURE_TIME_PARAM_LABEL = "--minExposureTimePerImage"
    m_edStringDEBUG_PARAM_LABEL             = "--DEBUG"
    m_edStringVERSION_PARAM_LABEL           = "--version or -v"
    m_edStringPLUGIN_PARAM_LABEL            = "--plugin"
    m_edStringTEMPLATE_PARAM_LABEL          = "--generateTemplate"
    m_edStringCOMPLEXITY_PARAM_LABEL        = "--complexity"
    m_edStringRESULTS_FILE_LABEL            = "--resultFile"
    m_edStringFORCED_SPACE_GROUP_LABEL      = "--forcedSpaceGroup"

    """
    Derives from EDApplicationDeprecated in order to add a Configuration Feature
    """
    def __init__( self, _strConfigurationFileName = None ):
        """
        """
        EDApplicationDeprecated.__init__( self, EDApplicationMXv1Characterisation.m_edStringAPPLICATION_NAME, \
                                EDApplicationMXv1Characterisation.m_edStringAPPLICATION_VERSION, \
                                _strConfigurationFileName )
        self.m_edStringPluginControlSubWedgeAssemble          = EDString( "EDPluginControlSubWedgeAssemblev10" )
        self.m_edPluginControlSubWedgeAssemble                = None
        self.m_edStringPluginControlCharacterisationName      = EDString( "EDPluginControlCharacterisationv10" )
        self.m_edPluginControlCharacterisation                = None
        self.m_edListImagePaths                               = None
        self.m_edStringDatasetFileName                        = None
        self.m_edStringApplicationWorkingDirectory            = None
        self.m_fFlux                                          = None
        self.m_fMaxExposureTimePerDataCollection              = 10000 # s, default prototype value
        self.m_fMinExposureTimePerImage                       = None
        self.m_fBeamSize                                      = None
        self.m_bTemplateMode                                  = False
        self.m_edStringGeneratedTemplateFile                  = None             
        self.m_edStringComplexity                             = EDString( "none" )
        self.m_edStringResultsFilePath                        = None
        self.m_edStringForcedSpaceGroup                       = None
        self.m_xsDataCollection                               = None
        self.m_bProcess                                       = True
        
    
    def preProcess( self ):
        """
        Processes the command line, creates the plugins
        """
        EDApplicationDeprecated.preProcess( self )
        EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.preProcess" )
        
        # Read command line parameters and check if they are ok
        self.m_bProcess = self.readAndProcessCommandLine()
        if ( self.m_bProcess == True ):
            # Check if XML data is given as input :
            if ( self.m_edStringDatasetFileName is not None ):
                try:
                    self.m_xsDataCollection = XSDataCollection.parseFile( self.m_edStringDatasetFileName )
                except:
                    errorMessage = EDMessage.ERROR_CANNOT_READ_FILE_02 % ( "EDApplicationMXv1Characterisation.process", self.m_edStringDatasetFileName )
                    EDVerbose.error( errorMessage )
                    raise RuntimeError, errorMessage
    
            self.createApplicationWorkingDirectory()
            
            self.m_edPluginControlSubWedgeAssemble = EDApplicationDeprecated.loadPlugin( self.m_edStringPluginControlSubWedgeAssemble )
            if( self.m_edPluginControlSubWedgeAssemble == None ):            
                errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ( "EDApplicationMXv1Characterisation.preProcess", self.m_edStringPluginControlSubWedgeAssemble )
                EDVerbose.error( errorMessage )       
                raise RuntimeError, errorMessage
            else:
                self.m_edPluginControlSubWedgeAssemble.setBaseDirectory( self.m_edStringApplicationWorkingDirectory )
                self.m_edPluginControlSubWedgeAssemble.setBaseName( "SubWedgeAssemble" )
    
            self.m_edPluginControlCharacterisation = EDApplicationDeprecated.loadPlugin( self.m_edStringPluginControlCharacterisationName )
            if( self.m_edPluginControlCharacterisation == None ):            
                errorMessage = EDMessage.ERROR_PLUGIN_NOT_LOADED_02 % ( "EDApplicationMXv1Characterisation.preProcess", self.m_edStringPluginControlCharacterisationName )
                EDVerbose.error( errorMessage )       
                raise RuntimeError, errorMessage
            else:
                self.m_edPluginControlCharacterisation.setBaseDirectory( self.m_edStringApplicationWorkingDirectory )
                self.m_edPluginControlCharacterisation.setBaseName( "Characterisation" )
            

    def process( self ):
        """
        If necessary, first assemble sub wedges from the image headers. 
        Then the characterisation plugin is executed if the generate template option is not activated.
        """
        EDApplicationDeprecated.process( self )
        EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.process" )

        if ( self.m_bProcess == True ):
            if( self.m_bTemplateMode == True ):
                self.m_edPluginControlSubWedgeAssemble.connectSUCCESS( self.generateTemplateFile )
            else:
                self.m_edPluginControlSubWedgeAssemble.connectSUCCESS( self.doSubWedgeAssembleSUCCESS )
            self.m_edPluginControlSubWedgeAssemble.connectFAILURE( self.doSubWedgeAssembleFAILURE )
            self.m_edPluginControlCharacterisation.connectSUCCESS( self.doCharacterisationSUCCESS )
            self.m_edPluginControlCharacterisation.connectFAILURE( self.doCharacterisationFAILURE )
    
            if ( self.m_xsDataCollection is None ):
                self.createDataCollectionFromImageHeaders( self.m_edPluginControlSubWedgeAssemble )
            else:
                self.runCharacterisationPlugin( self.m_edPluginControlCharacterisation )
            
            
    def createDataCollectionFromImageHeaders( self, _edPlugin ):    
        """
        """          
        EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.createDataCollectionFromImageHeaders" )
        xsDataInputSubWedgeAssemble = XSDataInputSubWedgeAssemble()
        for pyStrImagePath in self.m_edListImagePaths:
            xsDataFile = XSDataFile()
            xsDataFile.setPath( XSDataString( pyStrImagePath ) )
            xsDataInputSubWedgeAssemble.addFile( xsDataFile )
        self.m_edPluginControlSubWedgeAssemble.setDataInput( xsDataInputSubWedgeAssemble )
        self.m_edPluginControlSubWedgeAssemble.executeSynchronous()

    
    def runCharacterisationPlugin( self, _edPlugin = None ):
            # Run the characterisation plugin
            self.m_edPluginControlCharacterisation.setDataInput( self.m_xsDataCollection )
            self.m_edPluginControlCharacterisation.executeSynchronous()

    
    def generateTemplateFile( self, _edPlugin ):
        EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.generateTemplateFile" )
        self.createDataCollectionFromSubWedges()
        if( self.m_edStringGeneratedTemplateFile is None ):
            EDVerbose.screen( "No argument for command line --generateTemplate key word found!" )            
        elif ( self.m_xsDataCollection is None ):
            EDVerbose.screen( "ERROR! Cannot generate template file %s, please check the log files." % self.m_edStringGeneratedTemplateFile  )                    
        else:                
            EDVerbose.screen( "Generating xml template input file for edna: " + self.m_edStringGeneratedTemplateFile + "..." )
            self.m_xsDataCollection.outputFile( self.m_edStringGeneratedTemplateFile )
    
                
    def doSubWedgeAssembleSUCCESS( self, _edPlugin ):
        EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.doSubWedgeAssembleSUCCESS" )
        self.createDataCollectionFromSubWedges()
        self.runCharacterisationPlugin()

    def doSubWedgeAssembleFAILURE( self, _edPlugin ):
        EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.doSubWedgeAssembleFAILURE" )
        EDVerbose.screen( "Execution of " + self.m_edStringPluginControlSubWedgeAssemble + "  failed." )
        EDVerbose.screen( "Please inspect the log file for further information." )

    
    def doCharacterisationSUCCESS( self, _edPlugin ):
        EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.doCharacterisationSUCCESS" )
        # Store the results if requested
        if ( self.m_edStringResultsFilePath is not None ):
            xsDataCharacterisationResult = _edPlugin.getDataOutput()
            if ( xsDataCharacterisationResult is not None ):
                xsDataCharacterisationResult.outputFile( self.m_edStringResultsFilePath )
        if ( _edPlugin.getErrorMessages().getNumberObjects() != 0 ):
            self.doCharacterisationFAILURE( _edPlugin )
        elif ( _edPlugin.getWarningMessages().getNumberObjects() ):
            EDVerbose.screen( "Characterisation successful with warning messages, please check the log file." )
        else:
            EDVerbose.screen( "Characterisation successful!" )
        _edPlugin.verboseScreenExecutiveSummary()
              
                    
    def doCharacterisationFAILURE( self, _edObject ):
        EDVerbose.screen( "Characterisation failure!" )

    
    def readAndProcessCommandLine( self ):
        """
        Reads and processes the command line
        """
        EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.readAndProcessCommandLine" )
        bCommandLineIsOk = False
        # Check if m_edStringDATASET_PARAM_LABEL is given:
        self.m_edStringDatasetFileName = self.getCommandLineArgument( EDApplicationMXv1Characterisation.m_edStringDATASET_PARAM_LABEL )
        if ( self.m_edStringDatasetFileName is not None ):
            EDVerbose.screen( "Reading input data from : %s" % ( self.m_edStringDatasetFileName ) )
            EDVerbose.screen( "(Any other command line arguments are neglected.)" )
            bCommandLineIsOk = True
        else:
            # Special processing necessary for image list:
            EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.readAndProcessCommandLine: Searching for " + EDApplicationMXv1Characterisation.m_edStringIMAGE_PARAM_LABEL + " tag." )
            edStringCommandLine = self.getCommandLine()
            EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.readAndProcessCommandLine: Command line: " + edStringCommandLine )
            edListCommandLine = edStringCommandLine.split()
            # First go through in order to find if there are any images
            bFoundImageToken = False
            for pyStringCommandLineItem in edListCommandLine:
                edStringCommandLineItem = EDString( pyStringCommandLineItem )
                if( edStringCommandLineItem.equals( EDApplicationMXv1Characterisation.m_edStringTEMPLATE_PARAM_LABEL )):
                    self.m_bTemplateMode = True
                    self.m_edStringGeneratedTemplateFile = self.getCommandLineArgument( EDApplicationMXv1Characterisation.m_edStringTEMPLATE_PARAM_LABEL )
                    if ( self.m_edStringGeneratedTemplateFile == None ):
                        errorMessage = EDMessage.ERROR_MANDATORY_PARAM_MISSING_02 % ( "EDApplicationMXv1Characterisation.preProcess", "No argument for command line %s key word found!" % \
                                                                                      EDApplicationMXv1Characterisation.m_edStringTEMPLATE_PARAM_LABEL )
                        EDVerbose.error( errorMessage )       
                        raise RuntimeError, errorMessage
                    
                if ( bFoundImageToken == False):
                    if ( edStringCommandLineItem.equals( EDApplicationMXv1Characterisation.m_edStringIMAGE_PARAM_LABEL ) ):
                        bFoundImageToken = True
                else:
                    if ( EDString( edStringCommandLineItem[0:2] ).equals( "--" ) ):
                        bFoundImageToken = False
                    else:
                        if ( self.m_edListImagePaths is None ):
                            self.m_edListImagePaths = EDList()
                        edStringAbsolutePath = EDDiskExplorer.getAbsolutePath( edStringCommandLineItem )
                        EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.readAndProcessCommandLine: Found image path: " + edStringAbsolutePath )
                        self.m_edListImagePaths.add( edStringAbsolutePath )
            # Check that we have at least one image...
            if ( self.m_edListImagePaths is not None ):
                # Then find any other optional parameters
                edStringComplexity = self.getCommandLineArgument( EDApplicationMXv1Characterisation.m_edStringCOMPLEXITY_PARAM_LABEL )
                if ( edStringComplexity is not None ):
                    self.m_edStringComplexity = EDString( edStringComplexity )
                EDVerbose.screen( "BEST complexity set to                       : %s" % ( self.m_edStringComplexity ) )
                edStringFlux = self.getCommandLineArgument( EDApplicationMXv1Characterisation.m_edStringFLUX_PARAM_LABEL )
                if ( edStringFlux is not None ):
                    self.m_fFlux = EDString( edStringFlux ).atof()
                    EDVerbose.screen( "Beamline flux set to                         : %7.1f [photons/s]" % ( self.m_fFlux ) )
                edStringMaxExposureTimePerDataCollection = self.getCommandLineArgument( EDApplicationMXv1Characterisation.m_edStringMAX_EXPOSURE_TIME_PARAM_LABEL )
                if ( edStringMaxExposureTimePerDataCollection is not None ):
                    self.m_fMaxExposureTimePerDataCollection = EDString( edStringMaxExposureTimePerDataCollection ).atof()
                    EDVerbose.screen( "Max exposure time per data collection set to : %7.1f [s]" % ( self.m_fMaxExposureTimePerDataCollection ) )
                else:
                    EDVerbose.screen( "Max exposure time per data collection set to : %7.1f [s] (default value for the prototype)" % ( self.m_fMaxExposureTimePerDataCollection ) )                    
                edStringMinExposureTimePerImage = self.getCommandLineArgument( EDApplicationMXv1Characterisation.m_edStringMIN_EXPOSURE_TIME_PARAM_LABEL )
                if ( edStringMinExposureTimePerImage is not None ):
                    self.m_fMinExposureTimePerImage = EDString( edStringMinExposureTimePerImage ).atof()
                    EDVerbose.screen( "Minimum exposure time per image set to       : %7.3f [s]" % ( self.m_fMinExposureTimePerImage ) )
                edStringBeamSize = self.getCommandLineArgument( EDApplicationMXv1Characterisation.m_edStringBEAM_SIZE_PARAM_LABEL )
                if ( edStringBeamSize is not None ):
                    self.m_fBeamSize = EDString( edStringBeamSize ).atof()
                    EDVerbose.screen( "Beam size (beam assumed to be square)        : %7.3f [mm]" % ( self.m_fBeamSize ) )
                edStringForcedSpaceGroup = self.getCommandLineArgument( EDApplicationMXv1Characterisation.m_edStringFORCED_SPACE_GROUP_LABEL )
                if ( edStringForcedSpaceGroup is not None ):
                    self.m_edStringForcedSpaceGroup = EDString( edStringForcedSpaceGroup )
                    EDVerbose.screen( "Forced space group for indexing              : %s" % ( self.m_edStringForcedSpaceGroup ) )
                edStringResultsFilePath = self.getCommandLineArgument( EDApplicationMXv1Characterisation.m_edStringRESULTS_FILE_LABEL )
                if ( edStringResultsFilePath is not None ):
                    self.m_edStringResultsFilePath = EDString( edStringResultsFilePath )
                    EDVerbose.screen( "Results written to file                      : %s" % ( self.m_edStringResultsFilePath ) )
                bCommandLineIsOk = True
            else:
                self.usage()
                self.m_bProcess = False
        return bCommandLineIsOk
            
            
    def usage( self ):
        """
        Print usage...
        """
        EDVerbose.screen( "" )
        EDVerbose.screen( "Usage: " )
        EDVerbose.screen( "----- " )
        EDVerbose.screen( "" )
        EDVerbose.screen( " Either use:" )
        EDVerbose.screen( "" )
        EDVerbose.screen( " - the full/advanced XML input:" )
        EDVerbose.screen( "" )
        EDVerbose.screen( "   edna-prototype --data path_to_xsDataCollection_xml_file" )
        EDVerbose.screen( "" )
        EDVerbose.screen( " - or use the --image keyword (note that all these keywords are ignored if the --data option is used):" )
        EDVerbose.screen( "" )
        EDVerbose.screen( "   edna-prototype --image image1 [image2 image3 ...]" )
        EDVerbose.screen( "" )
        EDVerbose.screen( " In the case of using the --image command line keyword the following arguments are optional:" )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : BEST complexity [none (default), min or full]" % ( self.m_edStringCOMPLEXITY_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Beam flux [photon/s]" % ( self.m_edStringFLUX_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Beam size [mm] (beam assumed to be square)" % ( self.m_edStringBEAM_SIZE_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Max exposure time per data collection [s]" % ( self.m_edStringMAX_EXPOSURE_TIME_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Min exposure time per image [s]" % ( self.m_edStringMIN_EXPOSURE_TIME_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Forced space group for indexing" % ( self.m_edStringFORCED_SPACE_GROUP_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "-----------------------------------------------------------------------------------------------------------" )
        EDVerbose.screen( "" )
        EDVerbose.screen( " Additional options available:" )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Generates an xml template input file for edna" % ( self.m_edStringTEMPLATE_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Generates an xml file containing the characterisation results" % ( self.m_edStringRESULTS_FILE_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : DEBUG log traces" % ( self.m_edStringDEBUG_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Executable version info" % ( self.m_edStringVERSION_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        EDVerbose.screen( "%35s : Plugins connected to the application" % ( self.m_edStringPLUGIN_PARAM_LABEL ) )
        EDVerbose.screen( "" )
        

    def createApplicationWorkingDirectory( self ):
        EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.createApplicationWorkingDirectory" )
        edStringBaseDirectory = PyOs.environ.get("EDNA_BASE_DIRECTORY")
        if ( edStringBaseDirectory is None):
            EDVerbose.DEBUG( "*** EDApplicationMXv1Characterisation.createApplicationWorkingDirectory: Using current base directory as working directory." )
            edStringBaseDirectory = EDDiskExplorer.getCwd()
        edStringDateTime = EDTime.getTimeName()
        self.m_edStringApplicationWorkingDirectory = EDDiskExplorer.mergePath( edStringBaseDirectory, 
                                                                         "%s_%s-v%s" % \
                                                                         ( edStringDateTime, \
                                                                           EDApplicationMXv1Characterisation.m_edStringAPPLICATION_NAME, \
                                                                           EDApplicationMXv1Characterisation.m_edStringAPPLICATION_VERSION ) )
        # Check that a folder / file with the same name already exists
        if( EDDiskExplorer.existFolder( self.m_edStringApplicationWorkingDirectory ) or \
            EDDiskExplorer.existFile( self.m_edStringApplicationWorkingDirectory ) ):
            # It does exist so we have to modify the name of the working directory
            iIndex = 1
            bContinueFlag = True
            while ( bContinueFlag ):
                self.m_edStringApplicationWorkingDirectory = EDDiskExplorer.mergePath( edStringBaseDirectory, 
                                                                                        "%s_%s-v%s_%d" % \
                                                                                        ( edStringDateTime, \
                                                                                          EDApplicationMXv1Characterisation.m_edStringAPPLICATION_NAME, \
                                                                                          EDApplicationMXv1Characterisation.m_edStringAPPLICATION_VERSION, \
                                                                                          iIndex ) )
                if( EDDiskExplorer.existFolder( self.m_edStringApplicationWorkingDirectory ) or \
                    EDDiskExplorer.existFile( self.m_edStringApplicationWorkingDirectory ) ):
                    iIndex += 1
                else:
                    bContinueFlag = False
        # Make the directory
        EDDiskExplorer.createFolder( self.m_edStringApplicationWorkingDirectory )

        
                        
        
    def createDataCollectionFromSubWedges( self ):
        xsDataResultSubWedgeAssemble = self.m_edPluginControlSubWedgeAssemble.getDataOutput()
        self.m_xsDataCollection = XSDataCollection()
        # Default exposure time (for the moment, this value should be
        # possible to read from the command line)
        xsDataDiffractionPlan = XSDataDiffractionPlan()
        pyListSubWedge = xsDataResultSubWedgeAssemble.getSubWedge() 
        for xsDataSubWedge in pyListSubWedge:
            if ( self.m_edStringComplexity is not None ):
                xsDataDiffractionPlan.setComplexity( XSDataString( self.m_edStringComplexity ) )
            if ( self.m_fFlux is not None ):
                xsDataSubWedge.getExperimentalCondition().getBeam().setFlux( XSDataFloat( self.m_fFlux ) )
            if ( self.m_fBeamSize is not None ):
                xsDataSize = XSDataSize()
                xsDataSize.setX( XSDataLength( self.m_fBeamSize ) )
                xsDataSize.setY( XSDataLength( self.m_fBeamSize ) )
                xsDataSubWedge.getExperimentalCondition().getBeam().setSize( xsDataSize )
            if ( self.m_fMinExposureTimePerImage is not None ):
                xsDataSubWedge.getExperimentalCondition().getBeam().setMinExposureTimePerImage( XSDataFloat( self.m_fMinExposureTimePerImage ) )
        if ( self.m_edStringForcedSpaceGroup is not None ):
            xsDataDiffractionPlan.setForcedSpaceGroup( XSDataString( self.m_edStringForcedSpaceGroup ) )
        xsDataDiffractionPlan.setMaxExposureTimePerDataCollection( XSDataTime( self.m_fMaxExposureTimePerDataCollection ) )
        self.m_xsDataCollection.setSubWedge( pyListSubWedge )
        self.m_xsDataCollection.setDiffractionPlan( xsDataDiffractionPlan )
    
    
    